@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/examples/themes/metal.h   2.2.2   edited 2000-08-31 $
+** $Id: qt/examples/themes/metal.h   2.3.0   edited 2001-02-15 $
 **
 ** Definition of something or other
 **
@@ -27,6 +27,11 @@ public:
     void polish( QWidget* );
     void unPolish( QWidget* );
 
+    
+    void drawMetalButton( QPainter *p, int x, int y, int w, int h,
+			  bool sunken = FALSE, bool horz = TRUE );
+    
+    
     void drawButton( QPainter *p, int x, int y, int w, int h,
 			     const QColorGroup &g, bool sunken = FALSE,
 			     const QBrush *fill = 0 );
@@ -40,6 +45,23 @@ public:
 		    const QColorGroup &, bool sunken,
 		    int lineWidth, const QBrush *fill );
 
+    
+    void drawSlider( QPainter *p,
+		     int x, int y, int w, int h,
+		     const QColorGroup &g,
+		     Orientation orient, bool tickAbove, bool tickBelow );
+    
+    void drawScrollBarControls( QPainter* p, const QScrollBar* sb,
+				int sliderStart, uint controls,
+				uint activeControl );
+
+    
+    void drawComboButton( QPainter *p, int x, int y, int w, int h,
+			  const QColorGroup &g, bool sunken = FALSE,
+			  bool editable = FALSE,
+			  bool enabled = TRUE,
+			  const QBrush *fill = 0 );
+
 private:
     QPalette oldPalette;
 };
