@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/extensions/nsplugin/src/qnp.cpp   2.2.2   edited 2000-08-31 $
+** $Id: qt/extensions/nsplugin/src/qnp.cpp   2.3.0   edited 2001-02-09 $
 **
 ** Implementation of Qt extension classes for Netscape Plugin support.
 **
@@ -345,6 +345,12 @@ public:
 	    checkFocussedWidget();
 	}
 
+#ifdef _WS_WIN_
+        // Without this then widgets will not show in Windows at all
+        if ( event->type() == QEvent::Show || event->type() == QEvent::Paint )
+            QApplication::sendPostedEvents();
+#endif
+
 	return QApplication::notify( obj, event );
     }
 #endif
@@ -545,10 +551,7 @@ NPP_Shutdown(void)
 #endif
 	piApp = 0;
 
-	// delete qApp; ### Crashes under X11.  Waste memory until we can fix this.
-#ifdef _WS_WIN_
 	delete qApp;
-#endif
     }
 }
 
@@ -632,6 +635,11 @@ NPP_Destroy(NPP instance, NPSavedData** /*save*/)
 
 	if (This->widget) {
 	    This->widget->unsetWindow();
+#ifdef _WS_WIN_ // needed?
+	    if (This->window)
+		NPP_SetWindow( instance, 0 ); // unset
+#endif
+	    This->window = 0;
 	    delete This->widget;
 	}
 
@@ -671,8 +679,6 @@ NPP_SetWindow(NPP instance, NPWindow* window)
 	}
 #ifdef _WS_X11_
     } else if (This->window != (Window) window->window) {
-	if (This->window)
-	    NPP_SetWindow( instance, 0 ); // unset
 	This->window = (Window) window->window;
 #endif
 #ifdef _WS_WIN_
@@ -705,7 +711,11 @@ NPP_SetWindow(NPP instance, NPWindow* window)
 
 		// We are the first Qt-based plugin to arrive
 		new QApplication(This->display);
-		//XSynchronize(This->display,True);  // Helps debugging
+
+		// Helps debugging
+		//XSynchronize(This->display,True);
+		//XSetErrorHandler((int (*)(Display*dpy,XErrorEvent*))abort);
+
 		ASSERT(qt_np_count == 0);
 	    }
 	    installXtEventFilters(Safe);
@@ -1249,8 +1259,8 @@ public:
 	bool mt = hasMouseTracking();
 	bool hascurs = testWFlags( WState_OwnCursor );
 	QCursor curs = cursor();
-	clearWFlags( WState_Created );
-	clearWFlags( WState_Visible );
+	clearWState( WState_Created );
+	clearWState( WState_Visible );
 	create( 0, TRUE, FALSE );
 	setGeometry(g);
 	setBackgroundColor( bg );
@@ -1271,8 +1281,9 @@ void createNewWindowsForAllChildren(QWidget* parent, int indent=0)
 	QFixableWidget* c;
 	while ( (c = (QFixableWidget*)it.current()) ) {
 	    bool vis = c->isVisible();
-	    c->fix();
+	    // Fix children first, so propagation can work
 	    createNewWindowsForAllChildren(c,indent+1);
+	    c->fix();
 	    if ( vis ) c->show(); // Now that all children are valid.
 	    ++it;
 	}
@@ -1291,7 +1302,7 @@ void QNPWidget::setWindow(bool delold)
 
    if ( delold ) {
       // Make sure they get a show()
-      clearWFlags( WState_Visible );
+      clearWState( WState_Visible );
    }
 
 #ifdef _WS_X11_
