@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/extensions/xt/src/qxt.cpp   2.2.2   edited 2000-08-31 $
+** $Id: qt/extensions/xt/src/qxt.cpp   2.3.0   edited 2001-01-26 $
 **
 ** Implementation of Qt extension classes for Xt/Motif support.
 **
@@ -270,6 +270,7 @@ static void np_do_timers( void*, void* )
 	int interval = QMIN(tm->tv_sec,INT_MAX/1000)*1000 + tm->tv_usec/1000;
 	np_set_timer( interval );
     }
+    qxtapp->sendPostedEvents();
 }
 
 /*!
@@ -437,7 +438,7 @@ void QXtWidget::init(const char* name, WidgetClass widget_class,
   subwidgets, layouts, etc. using Qt functionality.
 */
 QXtWidget::QXtWidget(const char* name, Widget parent, bool managed) :
-    QWidget( 0, name, WResizeNoErase )
+    xtw(NULL), QWidget( 0, name, WResizeNoErase )
 {
     init(name, qWidgetClass, parent, 0, 0, 0, managed);
     Arg reqargs[20];
@@ -465,7 +466,7 @@ QXtWidget::QXtWidget(const char* name, Widget parent, bool managed) :
 QXtWidget::QXtWidget(const char* name, WidgetClass widget_class,
 		     QWidget *parent, ArgList args, Cardinal num_args,
 		     bool managed) :
-    QWidget( parent, name, WResizeNoErase )
+    xtw(NULL), QWidget( parent, name, WResizeNoErase )
 {
     if ( !parent )
 	init(name, widget_class, 0, 0, args, num_args, managed);
@@ -579,7 +580,7 @@ bool QXtWidget::isActiveWindow() const
  */
 void QXtWidget::moveEvent( QMoveEvent* )
 {
-    if ( xtparent )
+    if ( xtparent || !xtw )
 	return;
     XConfigureEvent c;
     c.type = ConfigureNotify;
@@ -598,7 +599,7 @@ void QXtWidget::moveEvent( QMoveEvent* )
  */
 void QXtWidget::resizeEvent( QResizeEvent* )
 {
-    if ( xtparent )
+    if ( xtparent || !xtw )
 	return;
     XtWidgetGeometry preferred;
     (void ) XtQueryGeometry( xtw, 0, &preferred );
