@@ -1,7 +1,13 @@
-#define QT_MODULE_ICONVIEW
-#define QT_MODULE_WORKSPACE
 #ifndef QT_H
 #endif // QT_H
+
+#define QT_MODULE_TOOLS
+#define QT_MODULE_KERNEL
+#define QT_MODULE_WIDGETS
+#define QT_MODULE_DIALOGS
+
+#define QT_MODULE_ICONVIEW
+#define QT_MODULE_WORKSPACE
 #define QT_MODULE_TABLE
 #define QT_MODULE_CANVAS
 #define QT_MODULE_XML
