@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qt/src/canvas/qcanvas.cpp   2.2.2   edited 2000-11-02 $
+** $Id: qt/src/canvas/qcanvas.cpp   2.3.0   edited 2001-02-24 $
 **
 ** Implementation of QCanvas and associated classes
 **
@@ -365,12 +365,12 @@ private:
 
 static int gcd(int a, int b)
 {
-    // ### Should use good method, but not speed critical.
-
-    int r = QMIN(a,b);
-    while ( a%r || b%r )
-	r--;
-    return r;
+    int r;
+    while ( (r = a%b) ) {
+	a=b;
+	b=r;
+    }
+    return b;
 }
 
 static int scm(int a, int b)
@@ -609,8 +609,8 @@ void QCanvas::resize(int w, int h)
   connects to this signal to keep the scrollview size correct.
 */
 
-/*!  Change the efficiency tuning parameters to \a mxclusers clusters,
-each of size \a chunksize (square).  This is a slow operation if you
+/*!  Change the efficiency tuning parameters to \a mxclusters clusters,
+each of size \a chunksze (square).  This is a slow operation if you
 have many objects on the canvas.
 
 Internally, a canvas uses a low-resolution "chunk matrix" to keep
@@ -667,8 +667,6 @@ void QCanvas::retune(int chunksze, int mxclusters)
 	for (QCanvasItem* item=hidden.first(); item != 0; item=hidden.next()) {
 	    item->show();
 	}
-
-	//oneone = tilew == tileh && tilew == chunksize;
     }
 }
 
@@ -767,6 +765,11 @@ viewing this QCanvas.  The QCanvasView class calls this.
 void QCanvas::addView(QCanvasView* view)
 {
     d->viewList.append(view);
+    if ( htiles>1 || vtiles>1 || pm.isNull() ) {
+	view->viewport()->setBackgroundColor(backgroundColor());
+    } else {
+	view->viewport()->setBackgroundPixmap(backgroundPixmap());
+    }
 }
 
 /*!
@@ -1155,14 +1158,13 @@ void QCanvas::removeItemFromChunkContaining(QCanvasItem* g, int x, int y)
     }
 }
 
-// ### warwick - either make it into a reimp, or complete the
-// unfinished setence.
-
 /*! Returns the color set by setBackgroundColor().
   By default, this is white.
 
-  Note that this function is not the same as
-  QWidget::backgroundColor().  The difference between them is
+  Note that this function is not a reimplementation
+  of QWidget::backgroundColor() (QCanvas is not a subclass
+  of QWidget), but all QCanvasViews that are viewing the
+  canvas will set their backgrounds to this
 
   \sa setBackgroundColor(), backgroundPixmap()
 */
@@ -1177,8 +1179,15 @@ QColor QCanvas::backgroundColor() const
 */
 void QCanvas::setBackgroundColor( const QColor& c )
 {
-    bgcolor = c;
-    setAllChanged();
+    if ( bgcolor != c ) {
+	bgcolor = c;
+	QCanvasView* view=d->viewList.first();
+	while (view != 0 ) {
+	    view->viewport()->setBackgroundColor(backgroundColor());
+	    view=d->viewList.next();
+	}
+	setAllChanged();
+    }
 }
 
 /*!  Returns the pixmap set by setBackgroundPixmap().  By default,
@@ -1199,6 +1208,11 @@ QPixmap QCanvas::backgroundPixmap() const
 void QCanvas::setBackgroundPixmap( const QPixmap& p )
 {
     setTiles(p, 1, 1, p.width(), p.height());
+    QCanvasView* view=d->viewList.first();
+    while (view != 0 ) {
+	view->viewport()->setBackgroundPixmap(backgroundPixmap());
+	view=d->viewList.next();
+    }
 }
 
 /*!  This virtual function is called for all updates of the QCanvas.
@@ -1277,8 +1291,6 @@ void QCanvas::setDoubleBuffering(bool y)
 }
 
 
-// ### warwick is the fourth paragraph correct?
-
 /*!  Sets the QCanvas to be composed of \a h tiles horizontally and \a
   v tiles vertically.  Each tile will be an image \a tilewidth by \a
   tileheight pixels from pixmap \a p.
@@ -1291,9 +1303,6 @@ void QCanvas::setDoubleBuffering(bool y)
   is repeated as necessary to cover the area.  If it is smaller, tiles
   to the right and bottom are not visible.
 
-  QCanvas has special optimizations for the case where the tiles are
-  square and the canvas is not larger than (h*tilewidth, v*tileheight).
-
   The width and height of \a p must be multipless of \a tilewidth and
   \a tileheight. If they are not, the action of this function is
   unspecified.
@@ -1317,7 +1326,6 @@ void QCanvas::setTiles( QPixmap p,
 	int s = scm(tilewidth,tileheight);
 	retune( s < 128 ? s : QMAX(tilewidth,tileheight) );
     }
-    //oneone = tilew == tileh && tilew == chunksize;
     setAllChanged();
 }
 
@@ -1748,18 +1756,12 @@ bool qt_testCollision(const QCanvasSprite* s1, const QCanvasSprite* s2)
 
     // s2image != 0
 
-    // ###
-    //
-    // A non-linear search would typically be more efficient.
-    // Optimal would be spiralling out from the center, but a simple
-    // vertical expansion from the centreline would suffice.
-    //
-    // My sister just had a baby 40 minutes ago, so I'm too
-    // brain-spun to implement it correctly!
-    //
+    // A non-linear search may be more efficient.
+    // Perhaps spiralling out from the center, or a simpler
+    // vertical expansion from the centreline.
 
-    // Let's make an assumption.  That sprite masks don't have
-    // bit orders for different machines!
+    // We assume that sprite masks don't have
+    // different bit orders.
     //
     // ASSERT(s1image->bitOrder()==s2image->bitOrder());
 
@@ -1889,7 +1891,7 @@ static bool collision_double_dispatch( const QCanvasSprite* s1,
   independently of the foreground and background, so a QCanvasSprite
   is just a image whose API makes it simpler to use animation and a
   hot spot.
-  
+
   QCanvasSprite draws very fast, at the cost of some memory.
 */
 
@@ -2266,15 +2268,7 @@ QCanvasPixmapArray::QCanvasPixmapArray()
 }
 
 /*!
-Constructs a QCanvasPixmapArray from files.
-
-The fc parameter sets the number of frames to be
-loaded for this image.  If \a fc is not 0, the
-filenames should contain a "%1",
-strings such as "foo%1.png".  The actual
-filenames are formed by replacing the %1 with each integer
-from 0 to fc-1, with leading zeroes sufficient for 4 digits.
-eg. foo0000.png, foo0001.png, foo0002.png, etc.
+Constructs a QCanvasPixmapArray from files. See readPixmaps() for details.
 */
 QCanvasPixmapArray::QCanvasPixmapArray(const QString& datafilenamepattern, int fc)
 {
@@ -2321,12 +2315,20 @@ void QCanvasPixmapArray::reset()
 }
 
 /*!
-  Read new pixmaps into the array.
+  Sets the array to \a fc pixmaps, read from files.
+
+  If \a fc is not 0, the
+  filenames should contain a "%1",
+  strings such as "foo%1.png".  The actual
+  filenames are formed by replacing the %1 with each integer
+  from 0 to fc-1, with leading zeroes sufficient for 4 digits.
+  eg. foo0000.png, foo0001.png, foo0002.png, etc.
 */
 bool QCanvasPixmapArray::readPixmaps(const QString& datafilenamepattern, int fc)
 {
     return readPixmaps(datafilenamepattern,fc,FALSE);
 }
+
 /*!
   When testing sprite collision detection
   the default is to use the image mask of the sprite.  By using
@@ -2387,9 +2389,18 @@ Returns the \a i-th pixmap in the array.
 /*!
 Replaces the \a i-th pixmap by \a p.  Note that \a p
 becomes owned by the array - it will be deleted later.
+
+If \a i is more than count()-1, the array is extended to make room.
 */
 void QCanvasPixmapArray::setImage(int i, QCanvasPixmap* p)
 {
+    if ( i >= framecount ) {
+	QCanvasPixmap** newimg = new QCanvasPixmap*[i+1];
+	memcpy(newimg, img, sizeof(newimg[0])*framecount);
+	framecount = i+1;
+	delete [] img;
+	img = newimg;
+    }
     delete img[i]; img[i]=p;
 }
 
@@ -2646,9 +2657,6 @@ QCanvasView::QCanvasView(QCanvas* canvas, QWidget* parent, const char* name, WFl
     viewing = 0;
     setCanvas(canvas);
 
-    if ( viewing )
-	viewport()->setBackgroundColor(viewing->backgroundColor());
-
     connect(this,SIGNAL(contentsMoving(int,int)),this,SLOT(cMoving(int,int)));
 }
 
@@ -2682,8 +2690,6 @@ void QCanvasView::setCanvas(QCanvas* canvas)
 	viewing->addView(this);
     }
     updateContentsSize();
-    if ( viewing )
-	viewport()->setBackgroundColor(viewing->backgroundColor());
 }
 
 void QCanvasView::updateContentsSize()
@@ -2769,7 +2775,7 @@ QCanvasPolygonalItem::QCanvasPolygonalItem(QCanvas* canvas) :
 }
 
 /*!
-  Destruct the QCanvasPolygonalItem.  Derived classes \e must 
+  Destruct the QCanvasPolygonalItem.  Derived classes \e must
   call hide() in their
   destructor, as this destructor cannot call the virtual methods.
 */
@@ -3101,7 +3107,11 @@ void QCanvasPolygon::moveBy(double dx, double dy)
     if ( idx || idy ) {
 	removeFromChunks();
 	poly.translate(idx,idy);
-	QCanvasItem::moveBy(dx,dy);
+    }
+    myx+=dx;
+    myy+=dy;
+    if ( idx || idy ) {
+	addToChunks();
     }
 }
 
@@ -3512,8 +3522,9 @@ void QCanvasEllipse::setAngles(int start, int length)
 QPointArray QCanvasEllipse::areaPoints() const
 {
     QPointArray r;
-    // ##### makeArc is not a very pretty ellipse
     r.makeArc(int(x()-w/2.0-1),int(y()-h/2.0-1),w+2,h+3,a1,a2);
+    r.resize(r.size()+1);
+    r.setPoint(r.size()-1,(int)x(),(int)y());
     return r;
 }
 
@@ -3592,11 +3603,8 @@ QRect QCanvasText::boundingRect() const { return brect; }
 
 void QCanvasText::setRect()
 {
-    static QWidget *w=0;
-    if (!w) w = new QWidget;
-    QPainter p(w);
-    p.setFont(fnt);
-    brect = p.boundingRect(QRect(int(x()),int(y()),0,0), flags, txt);
+    QFontMetrics fm(fnt);
+    brect = fm.boundingRect(int(x()), int(y()), 0, 0, flags, txt);
 }
 
 /*!
@@ -3696,10 +3704,16 @@ void QCanvasText::setColor(const QColor& c)
 */
 void QCanvasText::moveBy(double dx, double dy)
 {
-    if ( dx || dy ) {
+    int idx = int(x()+dx)-int(x());
+    int idy = int(y()+dy)-int(y());
+    if ( idx || idy ) {
 	removeFromChunks();
-	brect.moveBy((int)dx, (int)dy);
-	QCanvasItem::moveBy(dx,dy);
+	brect.moveBy(idx, idy);
+    }
+    myx+=dx;
+    myy+=dy;
+    if ( idx || idy ) {
+	addToChunks();
     }
 }
 
