@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qapplication.cpp   2.2.2   edited 2000-10-11 $
+** $Id: qt/src/kernel/qapplication.cpp   2.3.0   edited 2001-02-15 $
 **
 ** Implementation of QApplication class
 **
@@ -133,7 +133,7 @@
 
   </ul>
 
-  The <a href="simple-application.html">Application walk-trough
+  The <a href="simple-application.html">Application walk-through
   example</a> contains a typical complete main() that does the usual
   things with QApplication.
 
@@ -289,6 +289,7 @@ bool	  QApplication::fade_menu	= FALSE;
 bool	  QApplication::animate_combo	= FALSE;
 bool	  QApplication::animate_tooltip	= FALSE;
 bool	  QApplication::fade_tooltip	= FALSE;
+QApplication::Type qt_appType=QApplication::Tty;
 
 #if defined(QT_THREAD_SUPPORT)
 QMutex * QApplication::qt_mutex		= 0;
@@ -580,6 +581,7 @@ QApplication::QApplication( int &argc, char **argv, Type type )
 
 void QApplication::construct( int &argc, char **argv, Type type )
 {
+    qt_appType = type;
     qt_is_gui_used = (type != Tty);
     init_precmdline();
     static char *empty = (char*)"";
@@ -598,6 +600,11 @@ void QApplication::construct( int &argc, char **argv, Type type )
 
 }
 
+QApplication::Type QApplication::type() const
+{
+    return qt_appType;
+}
+
 
 #if defined(_WS_X11_)
 // note: #ifdef'ed stuff is NOT documented.
@@ -608,11 +615,19 @@ void QApplication::construct( int &argc, char **argv, Type type )
 
 QApplication::QApplication( Display* dpy )
 {
+    static int aargc = 1;
+    static char *aargv[] = { "unknown", 0 };
+
     qt_is_gui_used = TRUE;
     init_precmdline();
     // ... no command line.
     qt_init( dpy );
-    initialize( 0, 0 );
+
+#if defined(QT_THREAD_SUPPORT)
+    qt_mutex = new QMutex(TRUE);
+#endif
+
+    initialize( aargc, aargv );
 }
 
 QApplication::QApplication(Display *dpy, int argc, char **argv)
@@ -620,6 +635,11 @@ QApplication::QApplication(Display *dpy, int argc, char **argv)
     qt_is_gui_used = TRUE;
     init_precmdline();
     qt_init(dpy);
+
+#if defined(QT_THREAD_SUPPORT)
+    qt_mutex = new QMutex(TRUE);
+#endif
+
     initialize(argc, argv);
 }
 
@@ -658,13 +678,6 @@ void QApplication::initialize( int argc, char **argv )
 #endif
     is_app_running = TRUE; // no longer starting up
 
-#ifdef _WS_QWS_
-    // This is in qapplication.cpp in qt/main
-    extern QRect qt_maxWindowRect;
-    if ( qt_is_gui_used )
-	qt_maxWindowRect = qApp->desktop()->rect();
-#endif
-
 #ifndef QT_NO_STYLE
 #if defined(_WS_X11_)
     if ( qt_is_gui_used )
@@ -678,7 +691,7 @@ void QApplication::initialize( int argc, char **argv )
 #elif defined(_WS_X11_) && defined(_OS_IRIX_) && !defined(QT_NO_STYLE_SGI)
 	app_style = new QSGIStyle; // default comment
 #elif defined(_WS_X11_) && !defined(QT_NO_STYLE_MOTIF)
-	app_style = new QMotifStyle; // default style for X Windows
+	app_style = new QMotifStyle; // default style for X11
 #elif defined(_WS_MAC_) && !defined(QT_NO_STYLE_PLATINUM)
 	app_style = new QPlatinumStyle;
 #elif defined(_WS_QWS_) && !defined(QT_NO_STYLE_COMPACT)
@@ -686,7 +699,7 @@ void QApplication::initialize( int argc, char **argv )
 #elif !defined(QT_NO_STYLE_WINDOWS)
 	app_style = new QWindowsStyle; // default style for Windows
 #elif !defined(QT_NO_STYLE_MOTIF)
-	app_style = new QMotifStyle; // default style for X Windows
+	app_style = new QMotifStyle; // default style for X11
 #elif !defined(QT_NO_STYLE_PLATINUM)
 	app_style = new QPlatinumStyle;
 #else
@@ -2345,7 +2358,7 @@ void QApplication::setActiveWindow( QWidget* act )
 	if ( w->topLevelWidget() == old_active || w->topLevelWidget()==active_window ) {
 	    QColorGroup acg = w->palette().active();
 	    QColorGroup icg = w->palette().inactive();
-	    if ( acg != icg && 
+	    if ( acg != icg &&
 		 ( acg.background() != icg.background() ||
 		   acg.base() != icg.base() ||
 		   acg.button() != icg.button() ||
@@ -2422,7 +2435,7 @@ void Q_EXPORT qt_dispatchEnterLeave( QWidget* enter, QWidget* leave ) {
     if ( leave && !sameWindow ) {
 	w = leave;
 	do {
-	    leaveList.prepend( w );
+	    leaveList.append( w );
 	} while ( !w->isTopLevel() && (w = w->parentWidget() ) );
     }
     if ( enter && !sameWindow ) {
@@ -2457,7 +2470,7 @@ void Q_EXPORT qt_dispatchEnterLeave( QWidget* enter, QWidget* leave ) {
 
 	w = leave;
 	while ( w != wleave ) {
-	    leaveList.prepend( w );
+	    leaveList.append( w );
 	    w = w->parentWidget();
 	}
 	w = enter;
@@ -2614,6 +2627,10 @@ int QApplication::loopLevel() const
   Returns TRUE if the Qt library mutex is locked by a different thread,
   otherwise returns FALSE.
 
+  \e NOTE: Due to differing implementations of recursive mutexes on various
+  platforms, calling this function from the same thread that previous locked
+  the mutex will return undefined results.
+
   \sa lock(), unlock()
 */
 
