@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qdnd_x11.cpp   2.2.2   edited 2000-10-06 $
+** $Id: qt/src/kernel/qdnd_x11.cpp   2.3.0   edited 2001-02-10 $
 **
 ** XDND implementation for Qt.  See http://www.cco.caltech.edu/~jafl/xdnd/
 **
@@ -1100,7 +1100,9 @@ void QDragManager::move( const QPoint & globalPos )
 	// Ok.
     } else if ( target ) {
 	//me
-	target = qt_x11_findClientWindow( target, qt_wm_state, TRUE );
+	Window targetW = qt_x11_findClientWindow( target, qt_wm_state, TRUE );
+	if (targetW)
+	    target = targetW;
  	if ( qt_xdnd_deco && (!target || target == qt_xdnd_deco->winId()) ) {
  	    target = findRealWindow(globalPos,qt_xrootwin(),6);
  	}
