@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qfont.h   2.2.1   edited 2000-09-02 $
+** $Id: qt/src/kernel/qfont.h   2.3.0   edited 2001-02-01 $
 **
 ** Definition of QFont class
 **
@@ -50,7 +50,6 @@ struct QFontData;
 class QFontInternal;
 class QRenderedFont;
 
-
 class Q_EXPORT QFont					// font class
 {
 public:
@@ -86,6 +85,10 @@ public:
 		     Set_GBK,
 		     Set_Big5,
 
+		     TSCII,
+		     KOI8U,
+		     CP1251,
+		     PT154,
 		     /* The following are font-specific encodings that
 			we shouldn't need in a perfect world.
 		     */
@@ -95,9 +98,7 @@ public:
 		     JIS_X_0208 = 0xc0, Enc16 = JIS_X_0208,
 		     KSC_5601,
 		     GB_2312,
-		     Big5,
-		     TSCII,
-		     KOI8U
+		     Big5
     };
     enum StyleHint { Helvetica, Times, Courier, OldEnglish,  System, AnyStyle,
 		     SansSerif	= Helvetica,
@@ -221,6 +222,10 @@ private:
     friend class QFontMetrics;
     friend class QFontInfo;
     friend class QPainter;
+#if defined(_WS_X11_) && defined(QT_XFT)
+    friend void * qt_ft_font (const QFont *f);
+#endif
+
 #ifndef QT_NO_DATASTREAM
     friend Q_EXPORT QDataStream &operator<<( QDataStream &, const QFont & );
     friend Q_EXPORT QDataStream &operator>>( QDataStream &, QFont & );
