@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qfont_x11.cpp   2.2.2   edited 2000-11-06 $
+** $Id: qt/src/kernel/qfont_x11.cpp   2.3.0   edited 2001-03-05 $
 **
 ** Implementation of QFont, QFontMetrics and QFontInfo classes for X11
 **
@@ -57,6 +57,8 @@
 #include "qt_x11.h"
 #include "qmap.h"
 
+#include <netinet/in.h>
+
 // NOT REVISED
 
 /* UNICODE:
@@ -160,6 +162,9 @@ public:
     bool	    dirty() const;
     const char	   *name()  const;
     int		    xResolution() const;
+#ifdef QT_XFT
+    XftFont	    *ftfont() const;
+#endif
     XFontStruct	   *fontStruct() const;
     XFontSet	    fontSet() const;
     const QFontDef *spec()  const;
@@ -173,6 +178,9 @@ private:
     QCString	    n;
     XFontStruct	   *f;
     XFontSet	    set;
+#ifdef QT_XFT
+    XftFont	    *ft;
+#endif
     QFontDef	    s;
     int		    lw;
     int		    xres;
@@ -184,6 +192,10 @@ private:
 inline QFontInternal::QFontInternal( const QString &name )
     : n(name.ascii()), f(0), set(0), cmapper(0)
 {
+    f = 0;
+#ifdef QT_XFT
+    ft = 0;
+#endif
     s.dirty = TRUE;
 }
 
@@ -197,6 +209,13 @@ inline const char *QFontInternal::name() const
     return n;
 }
 
+#ifdef QT_XFT
+inline XftFont *QFontInternal::ftfont() const
+{
+    return ft;
+}
+#endif
+
 inline XFontStruct *QFontInternal::fontStruct() const
 {
     return f;
@@ -338,6 +357,8 @@ static struct {
     { "ta_TA.TSCII", QFont::TSCII },
     { "TSCII", QFont::TSCII },
     { "KOI8-U", QFont::KOI8U },
+    { "CP 1251", QFont::CP1251 },
+    { "PT 154", QFont::PT154 },
     { 0, /* anything */ QFont::ISO_8859_1 }
 };
 
@@ -385,6 +406,13 @@ void QFont::initialize()
     fontNameDict = new QFontNameDict( 29 );
     CHECK_PTR( fontNameDict );
     fontNameDict->setAutoDelete( TRUE );
+
+    QString vendorstring(ServerVendor(QPaintDevice::x11AppDisplay()));
+    int vendorrelease = VendorRelease(QPaintDevice::x11AppDisplay());
+
+    if (vendorstring.lower().contains("xfree86") &&
+	vendorrelease < 4000) {
+    }
 }
 
 /*!
@@ -452,21 +480,34 @@ HANDLE QFont::handle() const
     } else {
 	if ( d->fin->fontSet() )
 	    return 0;
-	if ( d->fin->fontStruct()->fid != last )
+	if ( d->fin->fontStruct () && d->fin->fontStruct()->fid != last )
 	    fontCache->find( d->fin->name() );
     }
 
+    if (d->fin->fontStruct())
     last = d->fin->fontStruct()->fid;
+    else
+	last = 1;
     return last;
 }
 
+#ifdef QT_XFT
+void *qt_ft_font (const QFont *f)
+{
+    if (!f->handle())
+	return 0;
+
+    return f->d->fin->ftfont();
+}
+#endif
+
 // ### maybe reorder the CharSet enum and get rid of this crap?
 static QFont::CharSet c8859[] = { QFont::ISO_8859_1,  QFont::ISO_8859_2,
 				  QFont::ISO_8859_3,  QFont::ISO_8859_4,
 				  QFont::ISO_8859_5,  QFont::ISO_8859_6,
 				  QFont::ISO_8859_7,  QFont::ISO_8859_8,
 				  QFont::ISO_8859_9,  QFont::ISO_8859_10,
-				  QFont::ISO_8859_9,  QFont::ISO_8859_12,
+				  QFont::ISO_8859_11,  QFont::ISO_8859_12,
 				  QFont::ISO_8859_13, QFont::ISO_8859_14,
 				  QFont::ISO_8859_15 };
 
@@ -502,7 +543,7 @@ static bool fillFontDef( const QCString &xlfd, QFontDef *fd,
 	if ( sscanf( tokens[CharsetEncoding], "%u", &tmp ) == 1 &&
 	     tmp >= 1 && tmp <= 15 ) {
 	    fd->charSet = c8859[tmp-1];
-	    if ( tmp == 8 || tmp == 6 ) { // hebrew and arabic
+	    if ( encodingName && ( tmp == 8 || tmp == 6 ) ) { // hebrew and arabic
 		*encodingName += "-I";
 	    }
 	}
@@ -531,6 +572,12 @@ static bool fillFontDef( const QCString &xlfd, QFontDef *fd,
 	fd->charSet = QFont::GB_2312;
     } else if( qstrncmp( tokens[CharsetRegistry], "big5", 4 ) == 0 ) {
 	fd->charSet = QFont::Big5;
+    } else if( qstrcmp( tokens[CharsetEncoding], "cp1251" ) == 0 ||
+  	       (qstrcmp( tokens[CharsetEncoding], "1251" ) == 0 ) ) {
+  	fd->charSet = QFont::CP1251;
+    } else if( qstrcmp( tokens[CharsetEncoding], "cp154" ) == 0 ||
+  	       (qstrcmp( tokens[CharsetEncoding], "154" ) == 0 ) ) {
+  	fd->charSet = QFont::PT154;
     } else {
 	fd->charSet = QFont::AnyCharSet;
     }
@@ -1074,6 +1121,44 @@ void QFont::initFontInfo() const
     f->computeLineWidth();
 
 
+#ifdef QT_XFT
+    XftFont *ft = d->fin->ft;
+    if ( ft ) {
+	resetFontDef( &f->s );
+	XftPattern *pattern = ft->pattern;
+	char *family;
+	int slant_value;
+	int weight_value;
+	int fixed_pitch;
+	XftPatternGetString (pattern, XFT_FAMILY, 0, &family);
+	XftPatternGetInteger (pattern, XFT_SLANT, 0, &slant_value);
+	XftPatternGetInteger (pattern, XFT_WEIGHT, 0, &weight_value);
+	XftPatternGetInteger (pattern, XFT_SPACING, 0, &fixed_pitch);
+	f->s.family = family;
+	f->s.pointSize = d->req.pointSize; //will fit because Xft fonts are scalable
+	f->s.charSet = QFont::Unicode;
+
+	if (weight_value == XFT_WEIGHT_LIGHT)
+	    weight_value = Light;
+	else if (weight_value == XFT_WEIGHT_DEMIBOLD)
+	    weight_value = DemiBold;
+	else if (weight_value < XFT_WEIGHT_BOLD)
+	    weight_value = Bold;
+	else if ( weight_value == XFT_WEIGHT_BLACK)
+	    weight_value = Black;
+	else
+	    weight_value = Normal;
+	f->s.weight = weight_value;
+	f->s.italic = slant_value != XFT_SLANT_ROMAN ? TRUE : FALSE;
+	f->s.fixedPitch = fixed_pitch != XFT_PROPORTIONAL ? TRUE : FALSE;
+	f->s.underline = d->req.underline;
+	f->s.strikeOut = d->req.strikeOut;
+	f->s.rawMode = FALSE;
+	f->s.dirty = FALSE;
+	return;
+    }
+#endif
+
     if (  d->exactMatch ) {
 	if ( PRIV->needsSet() ) {
 	    QCString locale = setlocale(LC_CTYPE, 0);
@@ -1107,13 +1192,13 @@ void QFont::initFontInfo() const
 		f->cmapper = QTextCodec::codecForLocale();
 #ifndef QT_NO_CODECS
 	} else if ( charSet() == JIS_X_0208 ) {
-	    f->cmapper = new QFontJis0208Codec;
+	    f->cmapper = QTextCodec::codecForName( "JIS_X_0208" );
 	} else if ( charSet() == KSC_5601 ) {
-	    f->cmapper = new QFontKsc5601Codec;
+	    f->cmapper = QTextCodec::codecForName( "KSC_5601" );
 	} else if ( charSet() == GB_2312 ) {
-	    f->cmapper = new QFontGB2312Codec;
+	    f->cmapper = QTextCodec::codecForName( "GB_2312" );
 	} else if ( charSet() == Big5 ) {
-	    f->cmapper = new QFontBig5Codec;
+	    f->cmapper = QTextCodec::codecForName( "Big5" );
 #endif //QT_NO_CODECS
 	} else {
 	    QCString encoding;
@@ -1133,13 +1218,13 @@ void QFont::initFontInfo() const
     if ( fillFontDef( f->name(), &f->s, &encoding ) ) { // valid XLFD?
 #ifndef QT_NO_CODECS
 	if ( encoding.left(9) == "jisx0208." ) {
-	    f->cmapper = new QFontJis0208Codec;
+	    f->cmapper = QTextCodec::codecForName( "JIS_X_0208" );
 	} else if ( encoding.left(8) == "ksc5601." ) {
-	    f->cmapper = new QFontKsc5601Codec;
+	    f->cmapper  = QTextCodec::codecForName( "KSC_5601" );
 	} else if ( encoding.left(7) == "gb2312." ) {
-	    f->cmapper = new QFontGB2312Codec;
+	    f->cmapper = QTextCodec::codecForName( "GB_2312" );
 	} else if ( encoding.left(4) == "big5" ) {
-	    f->cmapper = new QFontBig5Codec;
+	    f->cmapper = QTextCodec::codecForName( "Big5" );
 	} else
 #endif //QT_NO_CODECS
 	    f->cmapper = QTextCodec::codecForName( encoding );
@@ -1238,7 +1323,6 @@ static QCString setLocaleForCharSet(QFont::CharSet charset)
 /*!
   Loads the font.
 */
-
 void QFont::load() const
 {
     if ( !fontCache ) {				// not initialized
@@ -1250,6 +1334,101 @@ void QFont::load() const
     }
 
     QString k = key();
+
+#ifdef QT_XFT
+    if (qt_use_xft())
+    {
+	d->fin = fontCache->find (k);
+	if (!d->fin) {
+	    d->fin = fontDict->find (k);
+	    if (!d->fin) {
+		d->fin = new QFontInternal (k);
+		CHECK_PTR(d->fin);
+		fontDict->insert (k, d->fin);
+	    }
+	}
+	
+	XftFont	    *ft = d->fin->ft;
+	
+	if (!ft)
+	{
+	    const char	*family_value;
+	    const char	*generic_value;
+	    int	weight_value;
+	    int	slant_value;
+	    double	size_value;
+	    int	mono_value;
+
+	    family_value = family();
+	    weight_value = weight ();
+	    if (weight_value == 0)
+		weight_value = XFT_WEIGHT_MEDIUM;
+	    else if (weight_value < (Light + Normal) / 2)
+		weight_value = XFT_WEIGHT_LIGHT;
+	    else if (weight_value < (Normal + DemiBold) / 2)
+		weight_value = XFT_WEIGHT_MEDIUM;
+	    else if (weight_value < (DemiBold + Bold) / 2)
+		weight_value = XFT_WEIGHT_DEMIBOLD;
+	    else if (weight_value < (Bold + Black) / 2)
+		weight_value = XFT_WEIGHT_BOLD;
+	    else
+		weight_value = XFT_WEIGHT_BLACK;
+	    if (italic())
+		slant_value = XFT_SLANT_ITALIC;
+	    else
+		slant_value = XFT_SLANT_ROMAN;
+	    size_value = pointSize();
+	    mono_value = fixedPitch() ? XFT_MONO : XFT_PROPORTIONAL;
+	    
+	    int	family_width, family_height;
+	    if (sscanf (family_value, "%dx%d", 
+			&family_width, &family_height))
+	    {
+		family_value = "mono";
+		size_value = family_height;
+		mono_value = 1;
+	    }
+	    switch (styleHint()) {
+	    case SansSerif:
+	    default:
+		generic_value = "sans";
+		break;
+	    case Serif:
+		generic_value = "serif";
+		break;
+	    case TypeWriter:
+		generic_value = "mono";
+		mono_value = 1;
+		break;
+	    }
+#if 0
+	    printf ("%s,%s-%g:weight-%d:slant-%d\n",
+		    family_value, generic_value, size_value,
+		    weight_value, slant_value);
+#endif
+	    ft = XftFontOpen (QPaintDevice::x11AppDisplay(),
+			      QPaintDevice::x11AppScreen(),
+			      XFT_ENCODING, XftTypeString, "iso10646-1",
+			      XFT_FAMILY, XftTypeString, family_value,
+			      XFT_FAMILY, XftTypeString, generic_value,
+			      XFT_WEIGHT, XftTypeInteger, weight_value,
+			      XFT_SLANT, XftTypeInteger, slant_value,
+			      XFT_SIZE, XftTypeDouble, size_value,
+			      XFT_SPACING, XftTypeInteger, mono_value,
+			      0);
+#if 0
+	    if (ft)
+		XftPatternPrint (ft->pattern);
+#endif
+	    fontCache->insert (k, d->fin, 1);
+	    d->fin->ft = ft;
+	    d->exactMatch = TRUE;
+	    initFontInfo ();
+	}
+	return;
+    }
+#endif
+    
     QXFontName *fn = fontNameDict->find( k );
 
     if ( !fn ) {
@@ -1415,8 +1594,8 @@ int QFont_Private::fontMatchScore( char	 *fontName,	 QCString &buffer,
        else
 	       exactMatch = FALSE;
     } else if ( charSet() == KOI8U ) {
-       if ( strcmp( tokens[CharsetRegistry], "koi8" ) == 0 &&
-	    strcmp( tokens[CharsetEncoding], "u" ) == 0 )
+       if ( qstrcmp( tokens[CharsetRegistry], "koi8" ) == 0 &&
+	    qstrcmp( tokens[CharsetEncoding], "u" ) == 0 )
 	    score |= CharSetScore;
 	else
 	    exactMatch = FALSE;
@@ -1466,6 +1645,18 @@ int QFont_Private::fontMatchScore( char	 *fontName,	 QCString &buffer,
 	    score |= CharSetScore;
 	else
 	    exactMatch = FALSE;
+    } else if ( charSet() == CP1251 ) {
+        if  (qstrcmp( tokens[CharsetEncoding], "cp1251" ) == 0 ||
+                qstrcmp( tokens[CharsetEncoding], "1251" ) == 0)
+                  score |= CharSetScore;
+          else
+                  exactMatch = FALSE;
+    } else if ( charSet() == PT154 ) {
+        if  (qstrcmp( tokens[CharsetEncoding], "cp154" ) == 0 ||
+                qstrcmp( tokens[CharsetEncoding], "154" ) == 0)
+                  score |= CharSetScore;
+          else
+                  exactMatch = FALSE;
     } else if ( qstrcmp( tokens[CharsetRegistry], "iso10646" ) == 0 ) {
 	// Yes...
 	score |= CharSetScore;
@@ -1482,10 +1673,13 @@ int QFont_Private::fontMatchScore( char	 *fontName,	 QCString &buffer,
 	else
 	    exactMatch = FALSE;
     } else {
-	if ( pitch != 'p' )
+	if ( pitch == 'p' )
+	    score |= PitchScore;
+	else
 	    exactMatch = FALSE;
     }
 
+    // ### fix scaled bitmap fonts
     float diff;
     if ( *scalable ) {
 	diff = 9.0;	// choose scalable font over >= 0.9 point difference
@@ -1494,9 +1688,10 @@ int QFont_Private::fontMatchScore( char	 *fontName,	 QCString &buffer,
     } else {
 	int pSize;
 	float percentDiff;
+
 	pSize = ( 2*atoi( tokens[PointSize] )*atoi(tokens[ResolutionY]) +
-		  QPaintDevice::x11AppDpiY())
-		/ (QPaintDevice::x11AppDpiY() * 2); // adjust actual pointsize
+		      QPaintDevice::x11AppDpiY())
+		    / (QPaintDevice::x11AppDpiY() * 2); // adjust actual pointsize
 
 	if ( deciPointSize() != 0 ) {
 	    diff = (float)QABS(pSize - deciPointSize());
@@ -1506,7 +1701,7 @@ int QFont_Private::fontMatchScore( char	 *fontName,	 QCString &buffer,
 	    percentDiff = 100;
 	}
 
-	if ( percentDiff < 20 ) {
+	if ( percentDiff < 10 ) {
 	    score |= SizeScore;
 	    if ( pSize != deciPointSize() ) {
 		exactMatch = FALSE;
@@ -1604,12 +1799,17 @@ QCString QFont_Private::bestMatch( const char *pattern, int *score )
     QCString bestName;
     char *tokens[fontFields];
 
+    //qDebug("requesting font '%s' pixSize=%d pointSize=%d", pattern, pixelSize(), pointSize() );
+    //qDebug("best font: %d '%s'", best.score, best.name);
+    //qDebug("best scalable font: %d '%s'", bestScalable.score, bestScalable.name);
+
     if ( bestScalable.score > best.score ||
 	 bestScalable.score == best.score &&
 	 bestScalable.pointDiff < best.pointDiff ||
 	 bestScalable.score == best.score &&
 	 bestScalable.pointDiff == best.pointDiff &&
 	 bestScalable.weightDiff < best.weightDiff ) {
+	//qDebug("using scalable font");
 	qstrcpy( matchBuffer.data(), bestScalable.name );
 	if ( qParseXFontName( matchBuffer, tokens ) ) {
 	    int resx;
@@ -1646,7 +1846,7 @@ QCString QFont_Private::bestMatch( const char *pattern, int *score )
     bestName = best.name;
 
     XFreeFontNames( xFontNames );
-
+    //qDebug("best font found is '%s'", (const char *)bestName);
     return bestName;
 }
 
@@ -1713,8 +1913,26 @@ bool QFont_Private::fontmapping(const QString& filename)
 		chsetlst = QStringList::split( sep, s );
 	    } else {
 		QStringList familylst;
-		QRegExp sep( "[ =\t]" );
-		familylst = QStringList::split( sep, s );
+		QString ss = s.stripWhiteSpace();
+		int sslen = ss.length();
+		QString fontName;
+		bool quote = FALSE;
+		for( int i = 0; i <= sslen; ++i ) {
+		    if( i < sslen ) {
+			QChar c = ss[i];
+			if( c == '\"' || c == '\'' ) {
+			    quote = !quote;
+			    continue;
+			} else if( quote || (c != ' ' && c != '\t' && c != '=' ) ) {
+			    fontName += c;
+			    continue;
+			}
+		    }
+		    if ( !fontName.isEmpty() ) {
+			familylst << fontName;
+			fontName = "";
+		    }
+		}
 		if ( !chsetlst.isEmpty() && !familylst.isEmpty() ) {
 		    if ( !fontGuessingList )
 			fontGuessingList = new QList<FontGuessingPair>;
@@ -1762,7 +1980,7 @@ QCString QFont_Private::bestMatchFontSetMember( const QString& family,
 		    char s[1024];
 		    sprintf( s, "-*-%s-%s-%s-*-*-*-%d-%d-%d-*-*-%s,",
 				fontGuessingPair->family[i+1].latin1(),
-				wt, slant, size, xdpi, ydpi,
+			     wt, slant, size, xdpi, ydpi,
 				fontGuessingPair->charset[i].latin1() );
 				bestName.append(s);
 		}
@@ -1838,12 +2056,9 @@ QCString QFont_Private::findFont( bool *exact )
 		       "%s"
 		       "-*-*-*-%s-*-*-*-%d-%d-%d-*-*-*-*,"
 		       "-*-*-*-*-*-*-*-%d-%d-%d-*-*-*-*",
-		       foundry.ascii(), familyName.ascii(), wt, slant, size,
-		       xdpi, ydpi,
-		       foundry.ascii(), familyName.ascii(), slant, size
-		       , xdpi, ydpi,
-		       foundry.ascii(), familyName.ascii(), slant2, size,
-		       xdpi, ydpi,
+		       foundry.ascii(), familyName.ascii(), wt, slant, size, xdpi, ydpi,
+		       foundry.ascii(), familyName.ascii(), slant, size, xdpi, ydpi,
+		       foundry.ascii(), familyName.ascii(), slant2, size, xdpi, ydpi,
 		       familyName.ascii(), wt, slant, size, xdpi, ydpi,
 		       familyName.ascii(), slant, size, xdpi, ydpi,
 		       familyName.ascii(), slant2, size, xdpi, ydpi,
@@ -1948,6 +2163,17 @@ const QTextCodec *QFontMetrics::mapper() const
 
 #undef  FS
 #define FS (painter ? (XFontStruct*)fontStruct() : fin->fontStruct())
+
+#undef FT
+#undef FT_TYPE
+#ifdef QT_XFT
+#define FT_TYPE	XftFont
+#define FT (painter ? painter->cfont.handle(), (XftFont *)painter->cfont.d->fin->ftfont() : fin->ftfont())
+#else
+#define FT_TYPE void
+#define FT ((FT_TYPE *) 0)
+#endif
+
 #undef  SET
 #define SET ((XFontSet)fontSet())
 
@@ -1964,16 +2190,17 @@ int QFontMetrics::printerAdjusted(int val) const
 	 painter->device()->devType() == QInternal::Printer) {
 	painter->cfont.handle();
 	// ### printer font metrics hack
-	if ( painter->device() &&
-	     painter->device()->devType() == QInternal::Printer &&
-	     painter->cfont.d->printerHackFont ) {
-	    painter->cfont.d->printerHackFont->handle();
-	    val = val * painter->cfont.pointSize() / 64;
-	}
-	return val;
+// 	if ( painter->device() &&
+// 	     painter->device()->devType() == QInternal::Printer &&
+// 	     painter->cfont.d->printerHackFont ) {
+// 	    painter->cfont.d->printerHackFont->handle();
+// 	    val = val * painter->cfont.pointSize() / 64;
+// 	}
 	// this was wrong.
-	//int res = QPaintDevice::x11AppDpiY();
-	// return ( val * 72 + 36 ) / res; // PostScript is 72dpi
+	//return val;
+	int res = QPaintDevice::x11AppDpiY();
+	return ( val * 75 + 36 ) / res; // PostScript is 72dpi, but 75 gives a little more spacing as some
+	// fonts would be too close together otherwise.
     } else {
 	return val;
     }
@@ -1990,6 +2217,12 @@ int QFontMetrics::printerAdjusted(int val) const
 
 int QFontMetrics::ascent() const
 {
+#ifdef QT_XFT
+    XftFont	*ft = FT;
+
+    if (ft)
+	return printerAdjusted(ft->ascent);
+#endif
     XFontStruct *f = FS;
     if ( f )
 	return printerAdjusted(f->max_bounds.ascent);
@@ -2011,6 +2244,12 @@ int QFontMetrics::ascent() const
 
 int QFontMetrics::descent() const
 {
+#ifdef QT_XFT
+    XftFont	*ft = FT;
+
+    if (ft)
+	return printerAdjusted(ft->descent);
+#endif
     XFontStruct *f = FS;
     if ( f )
 	return printerAdjusted(f->max_bounds.descent - 1);
@@ -2018,35 +2257,64 @@ int QFontMetrics::descent() const
     return printerAdjusted(DESCENT(ext->max_ink_extent,ext->max_logical_extent));
 }
 
-inline bool inFont(XFontStruct *f, QChar ch)
+inline bool inFont(const QTextCodec *mapper, XFontStruct *f, QChar ch)
 {
+    XCharStruct *xcs = 0;
+
+    if ( mapper ) {
+	int l = 1;
+	QCString c = mapper->fromUnicode(ch,l);
+	if ( c.length() == 1 )
+	    ch = c[0];
+	else
+	    ch = QChar( *((ushort *)c.data()) ); // this *is* ugly
+    }
+
     if ( f->max_byte1 ) {
-	return ch.cell() >= f->min_char_or_byte2
-	    && ch.cell() <= f->max_char_or_byte2
-	    && ch.row() >= f->min_byte1
-	    && ch.row() <= f->max_byte1;
+	if ( ch.cell() >= f->min_char_or_byte2
+	     && ch.cell() <= f->max_char_or_byte2
+	     && ch.row() >= f->min_byte1
+	     && ch.row() <= f->max_byte1 )
+	    if( f->per_char )
+		xcs = f->per_char + ((ch.row() - f->min_byte1)
+				     * (f->max_char_or_byte2 - f->min_char_or_byte2 + 1)
+				     + ch.cell() - f->min_char_or_byte2);
+	    else
+		xcs = &f->max_bounds;
     } else if ( ch.row() ) {
 	uint ch16 = ch.unicode();
-	return ch16 >= f->min_char_or_byte2
-	    && ch16 <= f->max_char_or_byte2;
+	if ( ch16 >= f->min_char_or_byte2
+	     && ch16 <= f->max_char_or_byte2 )
+	    if( f->per_char )
+		xcs = f->per_char + ch16 - f->min_char_or_byte2;
+	    else
+		xcs = &f->max_bounds;
     } else {
-	return ch.cell() >= f->min_char_or_byte2
-	    && ch.cell() <= f->max_char_or_byte2;
+	if ( ch.cell() >= f->min_char_or_byte2
+	     && ch.cell() <= f->max_char_or_byte2 )
+	    if( f->per_char )
+		xcs = f->per_char + ch.cell() - f->min_char_or_byte2;
+	    else
+		xcs = &f->max_bounds;
     }
+    if ( !xcs || (xcs->width == 0 && xcs->ascent + xcs->descent == 0) )
+	return FALSE;
+    return TRUE;
 }
 
 /*!
   Returns TRUE if \a ch is a valid character in the font.
 */
-bool QFontMetrics::inFont(QChar ch) const
-{
+ bool QFontMetrics::inFont(QChar ch) const
+ {
+#ifdef QT_XFT
+     XftFont *ft = FT;
+    if (ft)
+	return XftGlyphExists (QPaintDevice::x11AppDisplay(), ft,
+			       (unsigned int) ch.unicode());
+#endif
     XFontStruct *f = FS;
-    if ( f && !mapper() ) {
-	return ::inFont( f, ch );
-    } else if ( mapper() ) {
-	return mapper()->canEncode(ch);
-    }
-    return TRUE; // ###### XFontSet range?
+    return ::inFont( mapper(), f, ch );
 }
 
 static
@@ -2060,41 +2328,137 @@ XCharStruct* charStr(const QTextCodec* mapper, XFontStruct *f, QChar ch)
     if ( mapper ) {
 	int l = 1;
 	QCString c = mapper->fromUnicode(ch,l);
-	// #### What if c.length()>1 ?
-	ch = c[0];
+	if ( c.length() == 1 )
+	    ch = c[0];
+	else
+	    ch = QChar( *((ushort *)c.data()) ); // this *is* ugly
     }
 
     if ( f->max_byte1 ) {
 	if ( !(ch.cell() >= f->min_char_or_byte2
-	    && ch.cell() <= f->max_char_or_byte2
-	    && ch.row() >= f->min_byte1
-	    && ch.row() <= f->max_byte1) )
+	       && ch.cell() <= f->max_char_or_byte2
+	       && ch.row() >= f->min_byte1
+	       && ch.row() <= f->max_byte1) ) {
 	    ch = QChar((ushort)f->default_char);
+	    if ( !(ch.cell() >= f->min_char_or_byte2
+		   && ch.cell() <= f->max_char_or_byte2
+		   && ch.row() >= f->min_byte1
+		   && ch.row() <= f->max_byte1) )
+		return f->per_char;
+	}
 	return f->per_char +
 	    ((ch.row() - f->min_byte1)
-		    * (f->max_char_or_byte2 - f->min_char_or_byte2 + 1)
-		+ ch.cell() - f->min_char_or_byte2);
+	     * (f->max_char_or_byte2 - f->min_char_or_byte2 + 1)
+	     + ch.cell() - f->min_char_or_byte2);
     } else if ( ch.row() ) {
 	uint ch16 = ch.unicode();
 	if ( !(ch16 >= f->min_char_or_byte2
-	    && ch16 <= f->max_char_or_byte2) )
+	       && ch16 <= f->max_char_or_byte2) ) {
 	    ch16 = f->default_char;
-	return f->per_char + ch16;
+	    if ( !(ch16 >= f->min_char_or_byte2
+		   && ch16 <= f->max_char_or_byte2) )
+		return f->per_char;
+	}
+	return f->per_char + ch16 - f->min_char_or_byte2;
     } else {
 	if ( !( ch.cell() >= f->min_char_or_byte2
-	    && ch.cell() <= f->max_char_or_byte2) )
+		&& ch.cell() <= f->max_char_or_byte2) ) {
 	    ch = QChar((uchar)f->default_char);
+	    if ( !( ch.cell() >= f->min_char_or_byte2
+		    && ch.cell() <= f->max_char_or_byte2) )
+		return f->per_char;
+	}
 	return f->per_char + ch.cell() - f->min_char_or_byte2;
     }
 }
 
+#ifdef QT_XFT
+static 
+XGlyphInfo *glyphStr(const QTextCodec* /*mapper*/, XftFont *ft, QChar ch)
+{
+    unsigned int	ich;
+    static XGlyphInfo	info;
+
+    ich = ch.unicode();
+    XftTextExtents32 (QPaintDevice::x11AppDisplay(),
+		      ft, &ich, 1, &info);
+#if 0
+    if (ich == 32)
+    printf ("glyphStr '%c' x: %d y: %d width: %d height: %d: xoff: %d yoff: %d\n",
+	    (char) ich, info.x, info.y, info.width, info.height,
+	    info.xOff, info.yOff);
+#endif
+    return &info;
+}
+#endif
+
 static void getExt( QString str, int len, XRectangle& ink,
-		    XRectangle& logical, XFontSet set, const QTextCodec* m )
+		    XRectangle& logical, XFontSet set, FT_TYPE *ft,
+		   const QTextCodec* m )
 {
     // Callers to this / this needs to be optimized.
     // Trouble is, too much caching in multiple clients will make the
     //  overall performance suffer.
-    
+
+#ifdef QT_XFT
+    if (ft)
+    {
+	XGlyphInfo	info;
+	unsigned short	*tmp;
+	unsigned short	*u, c;
+	int i;
+
+	tmp = new unsigned short[len];
+#if 0
+	printf ("getExt \"");
+#endif
+	// #####
+	// byteswap
+	u = (unsigned short *) str.unicode ();
+	for (i = 0; i < len; i++)
+	{
+	    c = u[i];
+#if 0
+	    printf ("%c", (char) (c >>8 ));
+#endif
+	    tmp[i] = htons( c );
+	}
+#if 0
+	printf ("\"\n");
+#endif
+	XftTextExtents16 (QPaintDevice::x11AppDisplay(),
+			  ft, tmp, len, &info);
+#if 0
+	if (len == 1 && tmp[0] == ' ')
+	{
+	    printf ("x: %d y: %d width: %d height: %d xoff: %d yoff: %d\n",
+		    info.x, info.y, info.width, info.height, info.xOff,
+		    info.yOff);
+	    printf ("font ascent: %d descent: %d max_advance_width: %d\n",
+		    ft->ascent, ft->descent, ft->max_advance_width);
+	}
+#endif
+	ink.x = -info.x;
+	ink.y = info.height - info.y;
+	ink.width = info.width;
+	ink.height = info.height;
+	logical.x = 0;
+	logical.y = -ft->ascent;
+	logical.width = info.xOff;
+	logical.height = ft->ascent + ft->descent;
+#if 0
+	if (len == 1 && tmp[0] == ' ')
+	{
+	    printf ("ink x: %d y: %d width: %d height: %d\n",
+		    ink.x, ink.y, ink.width, ink.height);
+	    printf ("log x: %d y: %d width: %d height: %d\n",
+		    logical.x, logical.y, logical.width, logical.height);
+	}
+#endif
+	delete [] tmp;
+	return;
+    }
+#endif
     QCString x = m->fromUnicode(str,len);
     XmbTextExtents( set, x, len, &ink, &logical );
 }
@@ -2114,12 +2478,17 @@ static void getExt( QString str, int len, XRectangle& ink,
 */
 int QFontMetrics::leftBearing(QChar ch) const
 {
+#ifdef QT_XFT
+    XftFont	*ft = FT;
+    if (ft)
+	return printerAdjusted(-glyphStr(mapper(),ft,ch)->x);
+#endif
     XFontStruct *f = FS;
     if ( f )
 	return printerAdjusted(charStr(mapper(),f,ch)->lbearing);
 
     XRectangle ink, log;
-    getExt(ch,1,ink,log,SET,mapper());
+    getExt(ch,1,ink,log,SET,FT,mapper());
     return printerAdjusted(LBEARING(ink,log));
 }
 
@@ -2137,13 +2506,28 @@ int QFontMetrics::leftBearing(QChar ch) const
 */
 int QFontMetrics::rightBearing(QChar ch) const
 {
+#ifdef QT_XFT
+    XftFont	*ft = FT;
+    if (ft)
+    {
+	XGlyphInfo  *info = glyphStr(mapper(),ft,ch);
+	int	    bearing;
+
+	bearing = info->xOff - (-info->x + info->width);
+#if 0	
+	printf ("xoff %d x %d width %d bearing %d\n", 
+		info->xOff, info->x, info->width, bearing);
+#endif
+	return printerAdjusted(bearing);
+    }
+#endif
     XFontStruct *f = FS;
     if ( f ) {
 	XCharStruct* cs = charStr(mapper(),f,ch);
 	return printerAdjusted(cs->width - cs->rbearing);
     }
     XRectangle ink, log;
-    getExt(ch,1,ink,log,SET,mapper());
+    getExt(ch,1,ink,log,SET,FT,mapper());
     return printerAdjusted(RBEARING(ink,log));
 }
 
@@ -2158,6 +2542,14 @@ int QFontMetrics::rightBearing(QChar ch) const
 */
 int QFontMetrics::minLeftBearing() const
 {
+#ifdef QT_XFT
+    XftFont	*ft = FT;
+    if (ft)
+    {
+	// ####
+	return 0;
+    }
+#endif
     // Don't need def->lbearing, the FS stores it.
     XFontStruct *f = FS;
     if ( f )
@@ -2178,6 +2570,14 @@ int QFontMetrics::minLeftBearing() const
 */
 int QFontMetrics::minRightBearing() const
 {
+#ifdef QT_XFT
+    XftFont	*ft = FT;
+    if (ft)
+    {
+	// ####
+	return 0;
+    }
+#endif
     // Safely cast away const, as we cache rbearing there.
     QFontDef* def = (QFontDef*)spec();
 
@@ -2218,6 +2618,11 @@ int QFontMetrics::minRightBearing() const
 
 int QFontMetrics::height() const
 {
+#ifdef QT_XFT
+    XftFont	*ft = FT;
+    if (ft)
+	return printerAdjusted(ft->ascent + ft->descent);
+#endif
     XFontStruct *f = FS;
     if ( f )
 	return printerAdjusted(f->max_bounds.ascent + f->max_bounds.descent);
@@ -2235,6 +2640,16 @@ int QFontMetrics::height() const
 
 int QFontMetrics::leading() const
 {
+#ifdef QT_XFT
+    XftFont	*ft = FT;
+    if (ft) {
+	int l = ft->height - (ft->ascent + ft->descent);
+	if (l > 0)
+	    return printerAdjusted(l);
+	else
+	    return 0;
+    }
+#endif
     XFontStruct *f = FS;
     if ( f ) {
 	int l = f->ascent		 + f->descent -
@@ -2261,6 +2676,11 @@ int QFontMetrics::leading() const
 
 int QFontMetrics::lineSpacing() const
 {
+#ifdef QT_XFT
+    XftFont	*ft = FT;
+    if (ft)
+	return ft->height;
+#endif
     return leading() + height();
 }
 
@@ -2287,12 +2707,18 @@ int QFontMetrics::lineSpacing() const
 
 int QFontMetrics::width( QChar ch ) const
 {
+#ifdef QT_XFT
+    XftFont	*ft = FT;
+    if (ft) {
+	return printerAdjusted(glyphStr(mapper(),ft,ch)->xOff);
+    }
+#endif
     XFontStruct *f = FS;
     if ( f ) {
 	return printerAdjusted(charStr(mapper(),f,ch)->width);
     } else {
 	XRectangle ink, log;
-	getExt(ch,1,ink,log,SET,mapper());
+	getExt(ch,1,ink,log,SET,FT,mapper());
 	return printerAdjusted(log.width);
     }
 }
@@ -2314,6 +2740,28 @@ int QFontMetrics::width( const QString &str, int len ) const
 {
     if ( len < 0 )
 	len = str.length();
+#ifdef QT_XFT
+    XftFont	*ft = FT;
+    if (ft) {
+	XGlyphInfo  info;
+	{
+	    unsigned short	tmp[1024];
+	    unsigned short	*u, c;
+	    int			i;
+
+	    // ### why byteswap
+	    u = (unsigned short *) str.unicode ();
+	    for (i = 0; i < len; i++)
+	    {
+		c = u[i];
+		tmp[i] = htons( c );
+	    }
+	    XftTextExtents16 (QPaintDevice::x11AppDisplay(), ft,
+			      tmp, len, &info);
+	}
+	return printerAdjusted(info.xOff);
+    }
+#endif
     XFontStruct *f = FS;
     if ( f ) {
 	const QTextCodec* m = mapper();
@@ -2325,7 +2773,7 @@ int QFontMetrics::width( const QString &str, int len ) const
 						 len ));
     }
     XRectangle ink, log;
-    getExt(str,len,ink,log,SET,mapper());
+    getExt(str,len,ink,log,SET,FT,mapper());
     return printerAdjusted(log.width);
 }
 
@@ -2371,6 +2819,42 @@ QRect QFontMetrics::boundingRect( const QString &str, int len ) const
 	strikeOut = strikeOutFlag();
     }
 
+#ifdef QT_XFT
+    XftFont	*ft = FT;
+    if (ft) {
+	XGlyphInfo  info;
+	{
+	    unsigned short	tmp[1024];
+	    unsigned short	*u, c;
+	    int			i;
+
+	    // ### why byteswap
+	    u = (unsigned short *) str.unicode ();
+	    for (i = 0; i < len; i++)
+	    {
+		c = u[i];
+		tmp[i] = htons( c );
+	    }
+	    XftTextExtents16 (QPaintDevice::x11AppDisplay(), ft,
+			      tmp, len, 
+			      &info);
+	}
+	overall.lbearing = -info.x;
+	overall.rbearing = info.width - info.x;
+	overall.ascent = info.height - info.y;
+	overall.descent = info.y;
+	overall.width = info.xOff;
+#if 0
+	qDebug ("lbearing %d rbearing %d ascent %d descent %d width %d",
+		overall.lbearing,
+		overall.rbearing,
+		overall.ascent,
+		overall.descent,
+		overall.width);
+#endif
+    } else
+#endif
+
     if ( f ) {
 	const QTextCodec *m = mapper();
 	if ( m ) {
@@ -2380,7 +2864,7 @@ QRect QFontMetrics::boundingRect( const QString &str, int len ) const
 	}
     } else {
 	XRectangle ink, log;
-	getExt(str,len,ink,log,SET,mapper());
+	getExt(str,len,ink,log,SET,FT,mapper());
 	overall.lbearing = LBEARING(ink,log);
 	overall.rbearing = ink.width+ink.x; // RBEARING(ink,log);
 	overall.ascent = ASCENT(ink,log);
@@ -2434,6 +2918,11 @@ QRect QFontMetrics::boundingRect( const QString &str, int len ) const
 
 int QFontMetrics::maxWidth() const
 {
+#ifdef QT_XFT
+    XftFont	*ft = FT;
+    if (ft)
+	return printerAdjusted(ft->max_advance_width);
+#endif
     XFontStruct *f = FS;
     if ( f )
 	return printerAdjusted(f->max_bounds.width);
@@ -2467,6 +2956,16 @@ int QFontMetrics::underlinePos() const
 
 int QFontMetrics::strikeOutPos() const
 {
+#ifdef QT_XFT
+    XftFont	*ft = FT;
+    if (ft) {
+	int pos = ft->ascent / 3;
+	if (pos)
+	    return printerAdjusted(pos);
+	else
+	    return 1;
+    }
+#endif
     XFontStruct *f = FS;
     if ( f ) {
 	int pos = f->max_bounds.ascent/3;
@@ -2600,8 +3099,7 @@ static bool fontExists( const QString &fontName )
 void QFontInternal::computeLineWidth()
 {
     char *tokens[fontFields];
-    QCString buffer(256);		// X font name always <= 255 chars
-    qstrcpy( buffer.data(), name() );
+    QCString buffer( name(), 255 );		// X font name always <= 255 chars
     if ( !qParseXFontName(buffer, tokens) ) {
 	lw   = 1;                   // name did not conform to X LFD
 	xres = QPaintDevice::x11AppDpiX();
