@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qnetworkprotocol.cpp   2.2.2   edited 2000-11-07 $
+** $Id: qt/src/kernel/qnetworkprotocol.cpp   2.3.0   edited 2001-01-26 $
 **
 ** Implementation of QNetworkProtocol class
 **
@@ -872,8 +872,7 @@ void QNetworkProtocol::removeMe()
 #ifdef QNETWORKPROTOCOL_DEBUG
 	qDebug( "QNetworkOperation:  autodelete of QNetworkProtocol %p", this );
 #endif
-	delete d->url;
-	d->url = 0;
+	delete d->url; // destructor deletes the network protocol
     }
 }
 
