@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qpainter.cpp   2.2.2   edited 2000-11-07 $
+** $Id: qt/src/kernel/qpainter.cpp   2.3.0   edited 2001-03-03 $
 **
 ** Implementation of QPainter, QPen and QBrush classes
 **
@@ -409,8 +409,8 @@ QPainter::QPainter()
 QPainter::QPainter( const QPaintDevice *pd )
 {
     init();
-    begin( pd );
-    flags |= CtorBegin;
+    if ( begin( pd ) )
+	flags |= CtorBegin;
 }
 
 
@@ -425,8 +425,8 @@ QPainter::QPainter( const QPaintDevice *pd,
 		    const QWidget *copyAttributes )
 {
     init();
-    begin( pd, copyAttributes );
-    flags |= CtorBegin;
+    if ( begin( pd, copyAttributes ) )
+	flags |= CtorBegin;
 }
 
 
@@ -2335,9 +2335,11 @@ void qt_format_text( const QFontMetrics& fm, int x, int y, int w, int h,
     int localTabStops = 0;	       		// tab stops
     if ( tabstops )
 	localTabStops = tabstops;
-    else
+    else {
 	localTabStops = fm.width(QChar('x'))*8;       	// default to 8 times x
-
+	if ( localTabStops == 0 )
+	    localTabStops = fm.maxWidth() * 8;
+    }
     QString word;
 
     bool fakeBreak = FALSE;
