@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qpngio.cpp   2.2.2   edited 2000-10-14 $
+** $Id: qt/src/kernel/qpngio.cpp   2.3.0   edited 2001-01-26 $
 **
 ** Implementation of PNG QImage IOHandler
 **
@@ -225,6 +225,20 @@ void setup_qt( QImage& image, png_structp png_ptr, png_infop info_ptr )
     }
 }
 
+
+#if defined(Q_C_CALLBACKS)
+extern "C" {
+#endif
+static void qt_png_warning(png_structp /*png_ptr*/, png_const_charp message)
+{
+    qWarning("libpng warning: %s", message);
+}
+
+#if defined(Q_C_CALLBACKS)
+}
+#endif
+
+
 static
 void read_png_image(QImageIO* iio)
 {
@@ -239,6 +253,8 @@ void read_png_image(QImageIO* iio)
 	return;
     }
 
+    png_set_error_fn(png_ptr, 0, 0, &qt_png_warning);
+
     info_ptr = png_create_info_struct(png_ptr);
     if (!info_ptr) {
 	png_destroy_read_struct(&png_ptr, 0, 0);
@@ -299,6 +315,9 @@ png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS)
     }
 #endif
 
+    image.setDotsPerMeterX(png_get_x_pixels_per_meter(png_ptr,info_ptr));
+    image.setDotsPerMeterY(png_get_y_pixels_per_meter(png_ptr,info_ptr));
+
     delete [] row_pointers;
 
     iio->setImage(image);
@@ -388,6 +407,8 @@ bool QPNGImageWriter::writeImage(const QImage& image, int quality, int off_x, in
 	return FALSE;
     }
 
+    png_set_error_fn(png_ptr, 0, 0, &qt_png_warning);
+
     info_ptr = png_create_info_struct(png_ptr);
     if (!info_ptr) {
 	png_destroy_write_struct(&png_ptr, 0);
@@ -467,6 +488,12 @@ bool QPNGImageWriter::writeImage(const QImage& image, int quality, int off_x, in
 	info_ptr->sig_bit.alpha = 8;
     }
 
+    // Swap ARGB to RGBA (normal PNG format) before saving on
+    // BigEndian machines
+    if ( QImage::systemByteOrder() == QImage::BigEndian ) {
+	png_set_swap_alpha(png_ptr);
+    }
+
     // Qt==ARGB==Big(ARGB)==Little(BGRA)
     if ( QImage::systemByteOrder() == QImage::LittleEndian ) {
 	png_set_bgr(png_ptr);
@@ -755,7 +782,7 @@ public:
 
 private:
     // Animation-level information
-    enum { MovieStart, FrameStart, Inside } state;
+    enum { MovieStart, FrameStart, Inside, End } state;
     int first_frame;
     int base_offx;
     int base_offy;
@@ -897,6 +924,7 @@ QPNGFormat::QPNGFormat()
     info_ptr = 0;
 }
 
+
 /*!
   Destructs a QPNGFormat.
 */
@@ -907,9 +935,6 @@ QPNGFormat::~QPNGFormat()
 }
 
 
-
-
-
 /*!
   This function decodes some data into image changes.
 
@@ -922,49 +947,52 @@ int QPNGFormat::decode(QImage& img, QImageConsumer* cons,
     image = &img;
 
     if ( state != Inside ) {
-       png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, 0, 0, 0);
-       png_set_compression_level(png_ptr, 9);
-       if (!png_ptr) {
-	  info_ptr = 0;
-	  image = 0;
-	  return 0;
-       }
+	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, 0, 0, 0);
+	if (!png_ptr) {
+	    info_ptr = 0;
+	    image = 0;
+		return -1;
+	}
 
-       info_ptr = png_create_info_struct(png_ptr);
-       if (!info_ptr) {
-	  png_destroy_read_struct(&png_ptr, &info_ptr, 0);
-	  image = 0;
-	  return 0;
-       }
+	png_set_error_fn(png_ptr, 0, 0, &qt_png_warning);
+	png_set_compression_level(png_ptr, 9);
 
-       if (setjmp((png_ptr)->jmpbuf)) {
-	  png_destroy_read_struct(&png_ptr, &info_ptr, 0);
-	  image = 0;
-	  return 0;
-       }
+	info_ptr = png_create_info_struct(png_ptr);
+	if (!info_ptr) {
+	    png_destroy_read_struct(&png_ptr, &info_ptr, 0);
+	    image = 0;
+	    return -1;
+	}
 
-       png_set_progressive_read_fn(png_ptr, (void *)this,
-	  info_callback, row_callback, end_callback);
+	if (setjmp((png_ptr)->jmpbuf)) {
+	    png_destroy_read_struct(&png_ptr, &info_ptr, 0);
+	    image = 0;
+	    return -1;
+	}
+
+	png_set_progressive_read_fn(png_ptr, (void *)this,
+	                            info_callback, row_callback, end_callback);
 
 #ifdef PNG_USER_CHUNK_SUPPORTED
-       png_set_user_chunk_fn(png_ptr, user_chunk_callback);
+	png_set_user_chunk_fn(png_ptr, user_chunk_callback);
 #endif
 
-       if ( state != MovieStart && *buffer != 0211 ) {
-	   // Good, no signature - the preferred way to concat PNG images.
-	   // Skip them.
-	   png_set_sig_bytes(png_ptr, 8);
-       }
+	if ( state != MovieStart && *buffer != 0211 ) {
+	    // Good, no signature - the preferred way to concat PNG images.
+	    // Skip them.
+	    png_set_sig_bytes(png_ptr, 8);
+	}
 
-       state = Inside;
+	state = Inside;
     }
 
     if ( !png_ptr ) return 0;
     
     if (setjmp(png_ptr->jmpbuf)) {
-       png_destroy_read_struct(&png_ptr, &info_ptr, 0);
-       image = 0;
-       return 0;
+	png_destroy_read_struct(&png_ptr, &info_ptr, 0);
+	image = 0;
+	state = MovieStart;
+	return -1;
     }
     unused_data = 0;
     png_process_data(png_ptr, info_ptr, (png_bytep)buffer, length);
@@ -988,7 +1016,7 @@ void QPNGFormat::info(png_structp png, png_infop)
 }
 
 void QPNGFormat::row(png_structp png, png_bytep new_row,
-   png_uint_32 row_num, int)
+    png_uint_32 row_num, int)
 {
     uchar* old_row = image->scanLine(row_num);
     png_progressive_combine_row(png, old_row, new_row);
