@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qrichtext.cpp   2.2.2   edited 2000-11-07 $
+** $Id: qt/src/kernel/qrichtext.cpp   2.3.0   edited 2001-02-27 $
 **
 ** Implementation of the internal Qt classes dealing with rich text
 **
@@ -139,6 +139,15 @@ bool QTextOptions::inSelection( const QtTriple& pos ) const
     return (pos >= t )  && (pos < selend);
 }
 
+struct QPixmapInt
+{
+    QPixmapInt() : ref( 0 ) {}
+    QPixmap pm;
+    int	    ref;
+};
+
+static QMap<QString, QPixmapInt> *pixmap_map = 0;
+
 QTextImage::QTextImage(const QMap<QString, QString> &attr, const QString& context,
 		       const QMimeSourceFactory &factory)
 {
@@ -149,45 +158,61 @@ QTextImage::QTextImage(const QMap<QString, QString> &attr, const QString& contex
 	height = attr["height"].toInt();
 
     reg = 0;
-    QImage img;
     QString imageName = attr["src"];
 
     if (!imageName)
 	imageName = attr["source"];
 
     if ( !imageName.isEmpty() ) {
-	const QMimeSource* m =
-			factory.data( imageName, context );
-	if ( !m ) {
-	    qWarning("QTextImage: no mimesource for %s", imageName.latin1() );
-	}
-	else {
-	    if ( !QImageDrag::decode( m, img ) ) {
-		qWarning("QTextImage: cannot decode %s", imageName.latin1() );
+	imgId = QString( "%1,%2,%3" ).arg( imageName ).arg( width ).arg( height );
+	if ( !pixmap_map )
+	    pixmap_map = new QMap<QString, QPixmapInt>;
+	if ( pixmap_map->contains( imgId ) ) {
+	    QPixmapInt& pmi = pixmap_map->operator[](imgId);
+	    pm = pmi.pm;
+	    pmi.ref++;
+	    width = pm.width();
+	    height = pm.height();
+	} else {
+	    QImage img;
+	    const QMimeSource* m =
+		factory.data( imageName, context );
+	    if ( !m ) {
+		qWarning("QTextImage: no mimesource for %s", imageName.latin1() );
+	    }
+	    else {
+		if ( !QImageDrag::decode( m, img ) ) {
+		    qWarning("QTextImage: cannot decode %s", imageName.latin1() );
+		}
+	    }
+	
+	    if ( !img.isNull() ) {
+		if ( width == 0 ) {
+		    width = img.width();
+		    if ( height != 0 ) {
+			width = img.width() * height / img.height();
+		    }
+		}
+		if ( height == 0 ) {
+		    height = img.height();
+		    if ( width != img.width() ) {
+			height = img.height() * width / img.width();
+		    }
+		}
+		
+		if ( img.width() != width || img.height() != height ){
+		    img = img.smoothScale(width, height);
+		    width = img.width();
+		    height = img.height();
+		}
+		pm.convertFromImage( img );
+	    }
+	    if ( !pm.isNull() ) {
+		QPixmapInt& pmi = pixmap_map->operator[](imgId);
+		pmi.pm = pm;
+		pmi.ref++;
 	    }
 	}
-    }
-
-    if ( !img.isNull() ) {
-	if ( width == 0 ) {
-	    width = img.width();
-	    if ( height != 0 ) {
-		width = img.width() * height / img.height();
-	    }
-	}
-	if ( height == 0 ) {
-	    height = img.height();
-	    if ( width != img.width() ) {
-		height = img.height() * width / img.width();
-	    }
-	}
-
-	if ( img.width() != width || img.height() != height ){
-	    img = img.smoothScale(width, height);
-	    width = img.width();
-	    height = img.height();
-	}
-	pm.convertFromImage( img );
 	if ( pm.mask() ) {
 	    QRegion mask( *pm.mask() );
 	    QRegion all( 0, 0, pm.width(), pm.height() );
@@ -210,6 +235,17 @@ QTextImage::QTextImage(const QMap<QString, QString> &attr, const QString& contex
 
 QTextImage::~QTextImage()
 {
+    if ( pixmap_map && pixmap_map->contains( imgId ) ) {
+	QPixmapInt& pmi = pixmap_map->operator[](imgId);
+	pmi.ref--;
+	if ( !pmi.ref ) {
+	    pixmap_map->remove( imgId );
+	    if ( pixmap_map->isEmpty() ) {
+		delete pixmap_map;
+		pixmap_map = 0;
+	    }
+	}
+    }
 }
 
 void QTextImage::realize( QPainter* p )
@@ -771,6 +807,7 @@ static void map_insert(QMap<QCString, QChar>* map, const char* str, const QChar&
     map->insert(str,ch);
 }
 
+// static ####################
 QMap<QCString, QChar> *htmlMap()
 {
     if ( !html_map ){
@@ -824,7 +861,10 @@ QChar QRichText::parseHTMLSpecialChar(const QString& doc, int& pos)
     pos++;
 
     if ( s.length() > 1 && s[0] == '#') {
-	return s.mid(1).toInt();
+	int num = s.mid(1).toInt();
+	if ( num == 151 ) // ### hack for designer manual
+	    return '-';
+	return num;
     }
 
     QMap<QCString, QChar>::Iterator it = htmlMap()->find(s);
@@ -1875,14 +1915,19 @@ void QRichTextFormatter::drawLine( QPainter* p, int ox, int oy,
     while ( current <= last ) {
 	QTextCharFormat *fmt  = format();
 	if ( !fmt->anchorHref().isEmpty() && fmt->useLinkColor() ) {
-	    p->setPen( ( QPaintDeviceMetrics(p->device()).depth() > 1) ? to.linkColor : cg.text() );
+	    if ( QPaintDeviceMetrics(p->device()).depth() > 1 ||
+		 p->device()->devType() == QInternal::Printer)
+		p->setPen( to.linkColor );
+	    else
+		p->setPen( cg.text() );
 	    if ( to.linkUnderline ) {
 		QFont f = fmt->font();
 		f.setUnderline( TRUE );
 		p->setFont( f );
 	    }
 	} else {
-	    if ( fmt->color().isValid() && ( QPaintDeviceMetrics(p->device()).depth() > 1) )
+	    if ( fmt->color().isValid() && ( QPaintDeviceMetrics(p->device()).depth() > 1 ||
+		p->device()->devType() == QInternal::Printer ) )
 		p->setPen( fmt->color() );
 	    else
 		p->setPen( cg.text() );
@@ -1964,7 +2009,7 @@ void QRichTextFormatter::drawLine( QPainter* p, int ox, int oy,
 	
 	}
 	if ( selected )
-	    p->setPen( ( QPaintDeviceMetrics(p->device()).depth() > 1) ? fmt->color() : cg.text() );
+	    p->setPen( ( isPrint || QPaintDeviceMetrics(p->device()).depth() > 1) ? fmt->color() : cg.text() );
 	
 	if ( isPrint && !printString.isEmpty() && !paragraph->text.haveSameFormat( current, current + 1) ) {
 	    p->drawText( printX, printY, printString );
@@ -2050,6 +2095,14 @@ void QRichTextFormatter::right( QPainter* p )
 {
     if ( !pastEnd() && !pastEndOfLine() ) {
 	QString c =  paragraph->text.charAt( current );
+#ifdef _WS_X11_
+	    //### workaround for broken courier fonts on X11
+	int l = c.length();
+	for ( int i = 0; i < l; i++ ) {
+	    if ( c[i] == QChar(0x00a0U) )
+		c[i] = ' ';
+	}
+#endif
 	if ( currentoffset  < int(c.length()) - 1 ) {
 	    QTextCharFormat* fmt = format();
 	    QFontMetrics fm( fmt->font() );
@@ -2070,6 +2123,14 @@ void QRichTextFormatter::left( QPainter* p )
 {
     if ( currentoffset > 0 ) {
 	QString c =  paragraph->text.charAt( current );
+#ifdef _WS_X11_
+	    //### workaround for broken courier fonts on X11
+	int l = c.length();
+	for ( int i = 0; i < l; i++ ) {
+	    if ( c[i] == QChar(0x00a0U) )
+		c[i] = ' ';
+	}
+#endif
 	QTextCharFormat* fmt = format();
 	QFontMetrics fm( fmt->font() );
 	if ( p ) {
@@ -2098,6 +2159,14 @@ void QRichTextFormatter::left( QPainter* p )
 	while ( current <  i - 1 )
 	    (void) rightOneItem( p );
 	QString c =  paragraph->text.charAt( current );
+#ifdef _WS_X11_
+	    //### workaround for broken courier fonts on X11
+	int l = c.length();
+	for ( int i = 0; i < l; i++ ) {
+	    if ( c[i] == QChar(0x00a0U) )
+		c[i] = ' ';
+	}
+#endif
 	if ( c.length() > 1 ) {
 	    currentoffset = c.length() - 1;
 	    QTextCharFormat* fmt = format();
@@ -2161,6 +2230,14 @@ void QRichTextFormatter::gotoNextItem( QPainter* p )
     }
     else {
 	QString c = item->c;
+#ifdef _WS_X11_
+	    //### workaround for broken courier fonts on X11
+	int l = c.length();
+	for ( int i = 0; i < l; i++ ) {
+	    if ( c[i] == QChar(0x00a0U) )
+		c[i] = ' ';
+	}
+#endif
 	QFontMetrics fm( formatinuse->font() );
 	if ( p )
 	    fm = p->fontMetrics();
