@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qthread_unix.cpp   2.2.2   edited 2000-10-08 $
+** $Id: qt/src/kernel/qthread_unix.cpp   2.3.0   edited 2001-02-20 $
 **
 ** QThread class for Unix
 **
@@ -35,8 +35,12 @@
 **
 **********************************************************************/
 
-#define _GNU_SOURCE
-#define __USE_UNIX98
+#include "qglobal.h"
+
+#if defined(_OS_LINUX_)
+# define _GNU_SOURCE
+# define __USE_UNIX98
+#endif
 
 #include "qthread.h"
 
@@ -169,7 +173,11 @@ void QMutex::unlock()
 
 
 /*!
-  Returns TRUE if the mutex is locked by another thread.
+  Returns TRUE if the mutex is locked by another thread and FALSE if not.
+
+  \e NOTE: Due to differing implementations of recursive mutexes on various
+  platforms, calling this function from the same thread that previous locked
+  the mutex will return undefined results.
 */
 bool QMutex::locked()
 {
@@ -355,7 +363,7 @@ void QThread::msleep( unsigned long msecs )
 
 
 /*!
-  System independent sleep.  This causes the current rhead to sleep for
+  System independent sleep.  This causes the current thread to sleep for
   \a usecs microseconds
 */
 void QThread::usleep( unsigned long usecs )
@@ -417,6 +425,7 @@ void QThread::exit()
   <li> The thread associated with this QThread object has finished
        execution (i.e. when it returns from run() ).  This
        function will return TRUE if the thread has finished.
+       It also returns TRUE if the thread has not been started yet.
   <li> \a time milliseconds has elapsed.  If \a time is ULONG_MAX (default
        argument), then the wait will never timeout (the thread must
        return from run() ).  This function will return FALSE
@@ -485,7 +494,7 @@ bool QThread::running() const
 /*!
   \class QWaitCondition qthread.h
   \brief The QWaitCondition class allows waiting/waking for conditions
-         between threads
+	 between threads
 
   \ingroup environment
 
@@ -740,7 +749,7 @@ QSemaphore::~QSemaphore()
 int QSemaphore::operator++(int)
 {
     int ret;
-    
+
     d->mutex.lock();
 
     while (d->value >= d->max)
@@ -774,7 +783,7 @@ int QSemaphore::operator--(int)
 
     d->cond.wakeAll();
     d->mutex.unlock();
-    
+
     return ret;
 }
 
@@ -793,7 +802,7 @@ int QSemaphore::operator+=(int n)
 
     while (d->value + n > d->max)
 	d->cond.wait(&(d->mutex));
-    
+
     d->value += n;
 
 #ifdef CHECK_RANGE
@@ -806,7 +815,7 @@ int QSemaphore::operator+=(int n)
     ret = d->value;
 
     d->mutex.unlock();
-    
+
     return ret;
 }
 
@@ -845,7 +854,7 @@ int QSemaphore::operator-=(int n)
 int QSemaphore::available() const {
     int ret;
 
-        d->mutex.lock();
+	d->mutex.lock();
     ret = d->max - d->value;
     d->mutex.unlock();
 
