@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qurl.cpp   2.2.2   edited 2000-09-13 $
+** $Id: qt/src/kernel/qurl.cpp   2.3.0   edited 2001-02-05 $
 **
 ** Implementation of QUrl class
 **
@@ -534,7 +534,7 @@ bool QUrl::parse( const QString& url )
 	{ 0,       Protocol,   0,          Path,       0,          0,          0,          0,         }, // Init
 	{ 0,       Protocol,   0,          0,          Separator1, 0,          0,          0,         }, // Protocol
 	{ 0,       Path,       Path,       Separator2, 0,          0,          0,          0,         }, // Separator1
-	{ 0,       Path,       0,          Separator3, 0,          0,          0,          0,         }, // Separator2
+	{ 0,       Path,       Path,          Separator3, 0,          0,          0,          0,         }, // Separator2
 	{ 0,       User,       User,       Separator3, Pass,       Host,       0,          0,         }, // Separator3
 	{ 0,       User,       User,       User,       Pass,       Host,       User,       User,      }, // User
 	{ 0,       Pass,       Pass,       Pass,       Pass,       Host,       Pass,       Pass,      }, // Pass
@@ -993,8 +993,11 @@ QString QUrl::dirPath() const
 {
     if ( path().isEmpty() )
 	return QString::null;
-
-    QString s = QFileInfo( path() ).dirPath();
+    
+    QString s = path();
+    // ### Why add a slash to the path before calling dirPath()??!
+    // s += "/";
+    s = QFileInfo( s ).dirPath( TRUE );
     if ( s[ (int)s.length() - 1 ] != '/' )
 	s += "/";
     return s;
