@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qwidget_x11.cpp   2.2.2   edited 2000-11-06 $
+** $Id: qt/src/kernel/qwidget_x11.cpp   2.3.0   edited 2001-02-27 $
 **
 ** Implementation of QWidget and QWindow classes for X11
 **
@@ -243,6 +243,9 @@ void QWidget::create( WId window, bool initializeWindow, bool destroyOldWindow)
 	}
 	setWinId( id );				// set widget id/handle + hd
     }
+#ifdef QT_XFT
+    qt_create_ft_draw (x11Display (), id, (Visual *) x11Visual (), x11Colormap());
+#endif
 
     if ( !topLevel ) {
 	if ( !testWFlags(WStyle_Customize) )
@@ -445,6 +448,9 @@ void QWidget::destroy( bool destroyWindow, bool destroySubWindows )
 	    qt_leave_modal( this );
 	else if ( testWFlags(WType_Popup) )
 	    qApp->closePopup( this );
+#ifdef QT_XFT
+	qt_destroy_ft_draw (x11Display (), winid);
+#endif
 	if ( testWFlags(WType_Desktop) ) {
 	    if ( acceptDrops() )
 		qt_dnd_enable( this, FALSE );
@@ -453,6 +459,10 @@ void QWidget::destroy( bool destroyWindow, bool destroySubWindows )
 		qt_XDestroyWindow( this, x11Display(), winid );
 	}
 	setWinId( 0 );
+
+	extern void qPRCleanup( QWidget *widget ); // from qapplication_x11.cpp
+	if ( testWState(WState_Reparented) )
+	    qPRCleanup(this);
     }
 }
 
@@ -492,11 +502,14 @@ void QWidget::reparent( QWidget *parent, WFlags f, const QPoint &p,
 	oldcurs = cursor();
 	unsetCursor();
     }
+    bool topdata_dnd = FALSE;
     bool accept_drops = acceptDrops();
     if ( accept_drops )
 	setAcceptDrops( FALSE ); // dnd unregister (we will register again below)
-    if ( isTopLevel() )
+    if ( isTopLevel() ) {
+	topdata_dnd = topData()->dnd;
 	topData()->dnd = 0; // unregister XDND atom
+    }
 
     QWidget* oldtlw = topLevelWidget();
     WId old_winid = winid;
@@ -527,7 +540,6 @@ void QWidget::reparent( QWidget *parent, WFlags f, const QPoint &p,
 	setWState( WState_ForceHide );	// new widgets do not show up in already visible parents
     create();
     const QObjectList *chlist = children();
-    QWidget* dndchild = 0; // remember one of the children accepting drops
     if ( chlist ) {				// reparent children
 	QObjectListIt it( *chlist );
 	QObject *obj;
@@ -537,8 +549,6 @@ void QWidget::reparent( QWidget *parent, WFlags f, const QPoint &p,
 		if ( !w->isTopLevel() ) {
 		    XReparentWindow( x11Display(), w->winId(), winId(),
 				     w->geometry().x(), w->geometry().y() );
-		    if ( w->acceptDrops() )
-			dndchild = w ;
 		} else if ( w->isPopup()
 			    || w->testWFlags(WStyle_DialogBorder)
 			    || w->testWFlags(WStyle_Dialog)
@@ -572,13 +582,12 @@ void QWidget::reparent( QWidget *parent, WFlags f, const QPoint &p,
 
     reparentFocusWidgets( oldtlw );
 
-
+    // re-register dnd
     if ( accept_drops )
 	setAcceptDrops( TRUE );
-    else if ( dndchild ) {
-	dndchild->clearWState(WState_DND); // reset this widget
-	dndchild->setAcceptDrops( TRUE ); // the new top-level will accept drops
-    }
+    if (topdata_dnd)
+	qt_dnd_enable(this, topdata_dnd);
+
     QCustomEvent e( QEvent::Reparent, 0 );
     QApplication::sendEvent( this, &e );
 }
@@ -1013,6 +1022,9 @@ void QWidget::setMouseTracking( bool enable )
   mouse when a button is pressed and keeps it until the last button is
   released.
 
+  Beware that only widgets actually shown on the screen may grab the
+  mouse input.
+
   \sa releaseMouse(), grabKeyboard(), releaseKeyboard()
 */
 
@@ -1176,11 +1188,16 @@ QWidget *QWidget::keyboardGrabber()
   focus.
 
   This function performs the same operation as clicking the mouse on
-  the title bar of a top-level window, at least on Windows. On X11,
-  the result depends on the Window Manager. If you want to ensure that
-  the window is stacked on top as well, call raise() in addition. Note
-  that the window has be to visible, otherwise setActiveWindow() has
-  no effect.
+  the title bar of a top-level window. On X11, the result depends on
+  the Window Manager. If you want to ensure that the window is stacked
+  on top as well, call raise() in addition. Note that the window has be
+  to visible, otherwise setActiveWindow() has no effect.
+
+  On Windows, if you are calling this when the application is not
+  currently the active one then it will not make it the active window.  It
+  will flash the task bar entry blue to indicate that the window has done
+  something.  This is due to Microsoft not allowing an application to
+  interrupt what the user is currently doing in another application.
 
   \sa isActiveWindow(), topLevelWidget(), show()
 */
@@ -1190,7 +1207,7 @@ void QWidget::setActiveWindow()
     QWidget *tlw = topLevelWidget();
     if ( tlw->isVisible() && !tlw->topData()->embedded ) {
 	XSetInputFocus( x11Display(), tlw->winId(), RevertToNone, qt_x_time);
-	
+
 #ifndef NO_XIM
 	if (tlw->topData()->xic)
 	    XSetICFocus( (XIC) tlw->topData()->xic);
@@ -1378,7 +1395,7 @@ void QWidget::showWindow()
 		XFree( (char *)h );
 	    topData()->showMode = sm == 1?3:0; // trigger reset to normal state next time
 	}
-	if ( topData()->parentWinId && topData()->parentWinId != qt_xrootwin() ) {
+	if ( topData()->parentWinId && topData()->parentWinId != qt_xrootwin() && !isMinimized() ) {
 	    qt_deferred_map_add( this );
 	    return;
 	}
@@ -1398,11 +1415,13 @@ void QWidget::hideWindow()
     deactivateWidgetCleanup();
     if ( isTopLevel() ) {
 	qt_deferred_map_take( this );
-	XWithdrawWindow( x11Display(), winId(), x11Screen() );
+	if ( winId() ) // in nsplugin, may be 0
+	    XWithdrawWindow( x11Display(), winId(), x11Screen() );
 	crect.moveTopLeft( fpos );
 	topData()->fsize = crect.size();
     } else {
-	XUnmapWindow( x11Display(), winId() );
+	if ( winId() ) // in nsplugin, may be 0
+	    XUnmapWindow( x11Display(), winId() );
     }
 }
 
@@ -2068,10 +2087,10 @@ int QWidget::metric( int m ) const
 		      DisplayHeight(dpy,scr);
 		break;
 	    case QPaintDeviceMetrics::PdmNumColors:
-		val = DisplayCells(dpy,scr);
+		val = x11Cells();
 		break;
 	    case QPaintDeviceMetrics::PdmDepth:
-		val = DisplayPlanes(dpy,scr);
+		val = x11Depth();
 		break;
 	    default:
 		val = 0;
