@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/network/qdns.cpp   2.2.2   edited 2000-10-06 $
+** $Id: qt/src/network/qdns.cpp   2.3.0   edited 2001-02-22 $
 **
 ** Implementation of QDns class.
 **
@@ -35,6 +35,13 @@
 **
 **********************************************************************/
 
+// 'u_char' and other BSD unsigned things used in <arpa> are undefined
+// on Tru64 4.0f unless...
+#if !defined(_OS_OSF_)
+// Has to appear before "qdns.h"??? Fix me!
+# define _OSF_SOURCE
+#endif
+
 #include "qdns.h"
 
 #ifndef QT_NO_DNS
@@ -146,7 +153,7 @@ public:
 
     void take( QDnsRR * );
 
-    void sweep();
+    void sweep( Q_UINT32 thisSweep );
 
     bool isEmpty() const { return rrs == 0 || rrs->isEmpty(); }
 
@@ -655,7 +662,10 @@ void QDnsAnswer::parse()
 	    }
 	    if ( rr ) {
 		rr->deleteTime = 0;
-		rr->expireTime = query->started + ttl;
+		if ( ttl > 0 )
+		    rr->expireTime = query->started + ttl;
+		else
+		    rr->expireTime = query->started + 20;
 		if ( rrno < ancount ) {
 		    answers++;
 		    rr->deleteTime = rr->expireTime;
@@ -921,7 +931,7 @@ QDnsManager::~QDnsManager()
 	globalManager = 0;
 }
 
-static Q_UINT32 lastSweep;
+static Q_UINT32 lastSweep = 0;
 
 void QDnsManager::cleanCache()
 {
@@ -936,7 +946,7 @@ void QDnsManager::cleanCache()
 
     while( (d=it.current()) != 0 ) {
 	++it;
-	d->sweep(); // after this, d may be empty
+	d->sweep( thisSweep ); // after this, d may be empty
 	if ( !again )
 	    again = !d->isEmpty();
     }
@@ -1038,6 +1048,7 @@ void QDnsManager::transmitQuery( int i )
 	QDnsAnswer answer( q );
 	answer.notify();
 	delete q;
+	QTimer::singleShot( 1000*10, QDnsManager::manager(), SLOT(cleanCache()) );
 	// and don't process anything more
 	return;
     }
@@ -1353,7 +1364,7 @@ QList<QDnsRR> * QDnsDomain::cached( const QDns * r )
 }
 
 
-void QDnsDomain::sweep()
+void QDnsDomain::sweep( Q_UINT32 thisSweep )
 {
     if ( !rrs )
 	return;
@@ -1362,7 +1373,7 @@ void QDnsDomain::sweep()
     rrs->first();
     while( (rr=rrs->current()) != 0 ) {
 	if ( !rr->deleteTime )
-	    rr->deleteTime = lastSweep; // will hit next time around
+	    rr->deleteTime = thisSweep; // will hit next time around
 
 #if defined(DEBUG_QDNS)
 	qDebug( "QDns::sweep: %s type %d expires %u %u - %s / %s",
@@ -1372,8 +1383,8 @@ void QDnsDomain::sweep()
 #endif
 	if ( rr->current == FALSE ||
 	     rr->t == QDns::None ||
-	     rr->deleteTime <= lastSweep ||
-	     rr->expireTime <= lastSweep )
+	     rr->deleteTime <= thisSweep ||
+	     rr->expireTime <= thisSweep )
 	    rrs->remove();
 	else
 	    rrs->next();
@@ -1962,7 +1973,11 @@ QString QDns::canonicalName() const
 #endif
 #include <resolv.h>
 
+#if defined (_OS_SOLARIS_) || defined (_OS_HPUX_)
+// According to changelog 23685, this was introduced to fix a problem under
+// Solaris. According to qt-bugs/arc-09/19264 it's needed for HPUX as well.
 extern "C" int res_init();
+#endif
 
 // if various defines aren't there, we'll set them safely.
 
