@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/network/qsocket.cpp   2.2.2   edited 2000-09-12 $
+** $Id: qt/src/network/qsocket.cpp   2.3.0   edited 2001-02-12 $
 **
 ** Implementation of QSocket class.
 **
@@ -110,8 +110,7 @@ QSocketPrivate::~QSocketPrivate()
 
   The functions you're likely to call most are connectToHost(),
   bytesAvailable(), canReadLine() and the ones it inherits from
-  QIODevice.  The <a href="linux-kernel.html">Linux Kernel Version
-  Walkthrough</a> uses them in a very simple program.
+  QIODevice.
 
   connectToHost() is the most-used function.  As its name implies, it
   opens a connection to a named host.
@@ -243,7 +242,7 @@ void QSocket::connectToHost( const QString &host, Q_UINT16 port )
     qDebug( "QSocket (%s)::connectToHost: host %s, port %d",
 	    name(), host.ascii(), port );
 #endif
-    setSocket( -1 );
+    setSocketIntern( -1 );
 
     d->state = HostLookup;
     d->host = host;
@@ -644,24 +643,28 @@ void QSocket::flush()
 		s = a ? a->size() : 0;
 	    }
 	    nwritten = d->socket->writeBlock( out.data(), i );
+	    if ( d->wsn )
+		d->wsn->setEnabled( FALSE ); // the QSocketNotifier documentation says so
 	} else {
 	    // Big block, write it immediately
 	    i = a->size() - d->windex;
 	    nwritten = d->socket->writeBlock( a->data() + d->windex, i );
+	    if ( d->wsn )
+		d->wsn->setEnabled( FALSE ); // the QSocketNotifier documentation says so
 	}
 	if ( nwritten ) {
-	    consumeWriteBuf( nwritten );
-	    consumed += nwritten;
+	    if ( consumeWriteBuf( nwritten ) )
+		consumed += nwritten;
 	}
 	if ( nwritten < i )
 	    osBufferFull = TRUE;
     }
     if ( consumed > 0 ) {
-	emit bytesWritten( consumed );
 #if defined(QSOCKET_DEBUG)
 	qDebug( "QSocket (%s): flush: wrote %d bytes, %d left",
 		name(), consumed, d->wsize );
 #endif
+	emit bytesWritten( consumed );
     }
     if ( d->state == Closing && d->wsize == 0 ) {
 #if defined(QSOCKET_DEBUG)
@@ -968,6 +971,14 @@ QString QSocket::readLine()
 
 void QSocket::sn_read()
 {
+    // Use alreadyCalled to avoid recursive calls of sn_read() (and as a result
+    // avoid emitting the readyRead() signal in a slot for readyRead(), if you
+    // use bytesAvailable()).
+    static bool alreadyCalled = FALSE;
+    if ( alreadyCalled )
+	return;
+    alreadyCalled = TRUE;
+
     char buf[4096];
     int  nbytes = d->socket->bytesAvailable();
     int  nread;
@@ -978,6 +989,7 @@ void QSocket::sn_read()
 	    tryConnection();
 	} else {
 	    // nothing to do, nothing to care about
+	    alreadyCalled = FALSE;
 	    return;
 	}
     }
@@ -1004,17 +1016,22 @@ void QSocket::sn_read()
 	    d->wba.clear();			// clear write buffer
 	    d->windex = d->wsize = 0;
 	    emit connectionClosed();
+	    alreadyCalled = FALSE;
 	    return;
 	} else {
 	    if ( nread < 0 ) {
 		if ( d->socket->error() == QSocketDevice::NoError ) {
 		    // all is fine
+		    alreadyCalled = FALSE;
 		    return;
 		}
-#if defined(CHECK_RANGE)
+#if defined(QSOCKET_DEBUG)
 		qWarning( "QSocket::sn_read (%s): Close error", name() );
 #endif
+		if (d->rsn)
+		    d->rsn->setEnabled( FALSE );
 		emit error( ErrSocketRead );	// socket close error
+		alreadyCalled = FALSE;
 		return;
 	    }
 	    a = new QByteArray( nread );
@@ -1039,15 +1056,23 @@ void QSocket::sn_read()
 	    }
 	}
 	if ( nread < 0 ) {
+	    if ( d->socket->error() == QSocketDevice::NoError ) {
+		// all is fine
+		alreadyCalled = FALSE;
+		return;
+	    }
 #if defined(CHECK_RANGE)
 	    qWarning( "QSocket::sn_read: Read error" );
 #endif
 	    delete a;
+	    if (d->rsn)
+		d->rsn->setEnabled( FALSE );
 	    emit error( ErrSocketRead );	// socket read error
+	    alreadyCalled = FALSE;
 	    return;
 	}
 	if ( nread != (int)a->size() ) {		// unexpected
-#if defined(CHECK_RANGE)
+#if defined(CHECK_RANGE) && !defined(_OS_WIN32_)
 	    qWarning( "QSocket::sn_read: Unexpected short read" );
 #endif
 	    a->resize( nread );
@@ -1056,6 +1081,8 @@ void QSocket::sn_read()
     d->rba.append( a );
     d->rsize += nread;
     emit readyRead();
+
+    alreadyCalled = FALSE;
 }
 
 
@@ -1078,6 +1105,8 @@ void QSocket::tryConnection()
 	qDebug( "QSocket (%s): sn_write: Got connection to %s",
 		name(), peerName().ascii() );
 #endif
+	if ( d->rsn )
+	    d->rsn->setEnabled( TRUE );
 	emit connected();
     } else {
 	d->state = Idle;
@@ -1098,7 +1127,6 @@ int QSocket::socket() const
     return d->socket->socket();
 }
 
-
 /*!
   Sets the socket to use \a socket and the state() to \c Connected. The socket
   should already be connected.
@@ -1108,44 +1136,57 @@ int QSocket::socket() const
 */
 
 void QSocket::setSocket( int socket )
+{
+    setSocketIntern( socket );
+    d->state = Connection;
+    d->rsn->setEnabled( TRUE );
+}
+
+
+/*!
+  Sets the socket to \a socket. This is both used by setSocket() and
+  connectToHost() and can be used also on unconnected sockets.
+*/
+
+void QSocket::setSocketIntern( int socket )
 {
     QSocketDevice *sd;
     if ( socket >= 0 )
-	sd = new QSocketDevice( socket, QSocketDevice::Stream );
+        sd = new QSocketDevice( socket, QSocketDevice::Stream );
     else
-	sd = new QSocketDevice( QSocketDevice::Stream );
-
+        sd = new QSocketDevice( QSocketDevice::Stream );
+ 
     if ( state() != Idle )
-	close();
+        close();
     // close may not have actually deleted the thing.  so, we brutally
     // Act.
     delete d;
-
+ 
     d = new QSocketPrivate( this );
     d->socket = sd;
     d->socket->setBlocking( FALSE );
     d->socket->setAddressReusable( TRUE );
-    d->state = Connection;
+    d->state = Idle;
     d->rsn = new QSocketNotifier( d->socket->socket(), QSocketNotifier::Read,
-				  this, "read" );
+                                  this, "read" );
     d->wsn = new QSocketNotifier( d->socket->socket(), QSocketNotifier::Write,
-				  this, "write" );
+                                  this, "write" );
     connect( d->rsn, SIGNAL(activated(int)), SLOT(sn_read()) );
-    d->rsn->setEnabled( TRUE );
+    d->rsn->setEnabled( FALSE );
     connect( d->wsn, SIGNAL(activated(int)), SLOT(sn_write()) );
     d->wsn->setEnabled( FALSE );
     // Initialize the IO device flags
     setFlags( IO_Direct );
     setStatus( IO_Ok );
     open( IO_ReadWrite );
-
+ 
     // hm... this is not very nice.
     d->host = QString::null;
     d->port = 0;
 #ifndef QT_NO_DNS
     delete d->dns;
     d->dns = 0;
-#endif
+#endif                                                                                                                                                                                
 }
 
 
