@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/opengl/qgl_x11.cpp   2.2.2   edited 2000-11-03 $
+** $Id: qt/src/opengl/qgl_x11.cpp   2.3.0   edited 2001-01-26 $
 **
 ** Implementation of OpenGL classes for Qt
 **
@@ -58,7 +58,7 @@
 
 
 /*
-  The create_cmap function is internal and used by QGLWidget::setContext()
+  The choose_cmap function is internal and used by QGLWidget::setContext()
   and GLX (not Windows).  If the application can't find any sharable
   colormaps, it must at least create as few colormaps as possible.  The
   dictionary solution below ensures only one colormap is created per visual.
@@ -103,7 +103,8 @@ static Colormap choose_cmap( Display *dpy, XVisualInfo *vi )
     if ( !cmap_dict ) {
 	cmap_dict = new QIntDict<CMapEntry>;
 	const char *v = glXQueryServerString( dpy, vi->screen, GLX_VERSION );
-	mesa_gl = strstr(v,"Mesa") != 0;
+	if ( v )
+	    mesa_gl = strstr(v,"Mesa") != 0;
 	qAddPostRoutine( cleanup_cmaps );
     }
 
@@ -155,7 +156,6 @@ static Colormap choose_cmap( Display *dpy, XVisualInfo *vi )
 			x->cmap = c[i].colormap;
 			x->scmap = c[i];
 			//qDebug( "Using RGB_DEFAULT scmap" );
-
 		    }
 		    i++;
 		}
@@ -923,5 +923,4 @@ bool QGLWidget::renderCxPm( QPixmap* pm )
     return TRUE;
 }
 
-#endif // GLX
-
+#endif
