@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/tools/qcstring.h   2.2.2   edited 2000-08-25 $
+** $Id: qt/src/tools/qcstring.h   2.3.0   edited 2001-03-02 $
 **
 ** Definition of the extended char array operations,
 ** and QByteArray and QCString classes
@@ -43,12 +43,12 @@
 #include "qarray.h"
 #endif // QT_H
 
-#include <string.h>
-
-#if defined(_OS_SUN_) && defined(_CC_GNU_)
+#if (defined(_OS_SUN_) && defined(_CC_GNU_)) || defined(_OS_LINUX_)
 #include <strings.h>
 #endif
 
+#include <string.h>
+
 
 /*****************************************************************************
   Fixes and workarounds for some platforms
