@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/tools/qfeatures.h   2.2.2   edited 2000-11-08 $
+** $Id: qt/src/tools/qfeatures.h   2.3.0   edited 2001-02-13 $
 **
 ** Global feature selection
 **
@@ -42,26 +42,13 @@
     ...
 */
 
-// Qt ships with a number of pre-defined configurations. If none suit
-// your needs, define QCONFIG_LOCAL and create a "qconfig-local.h" file.
-//
 // Note that disabling some features will produce a libqt that is not
 // compatible with other libqt builds. Such modifications are only
 // supported on Qt/Embedded where reducing the library size is important
 // and where the application-suite is often a fixed set.
 //
-#if defined(QCONFIG_LOCAL)
-#include <qconfig-local.h>
-#elif defined(QCONFIG_QPE)
-#include <qconfig-qpe.h>
-#elif defined(QCONFIG_MINIMAL)
-#include <qconfig-minimal.h>
-#elif defined(QCONFIG_SMALL)
-#include <qconfig-small.h>
-#elif defined(QCONFIG_MEDIUM)
-#include <qconfig-medium.h>
-#elif defined(QCONFIG_LARGE)
-#include <qconfig-large.h>
+#if defined(QCONFIG)
+#include "qconfig-" QCONFIG ".h"
 #else // everything...
 #include <qconfig.h>
 #endif
@@ -186,11 +173,11 @@
 
 // Fonts
 /*!
-    TrueType font files
-    <p>Scalable font format common on MS-Windows and becoming common on Unix.
+    Freetype font engine
+    <p>Scalable font formats supported by the Freetype engine.
     <p>Only supported on Qt/Embedded.
 */
-//#define QT_NO_TRUETYPE
+//#define QT_NO_FREETYPE
 /*!
     BDF font files
     <p>The Bitmap Distribution Format (BDF) font file format, common
@@ -323,10 +310,10 @@
 # define QT_NO_QWS_MULTIPROCESS
 #endif
 
-#if defined(QT_NO_QWS_MULTIPROCESS) || defined(QT_NO_DATASTREAM)
-    /*!
-      Palmtop Communication Protocol
-    */
+#if defined(QT_NO_DATASTREAM)
+/*!
+  Palmtop Communication Protocol
+*/
 # define QT_NO_COP
 #endif
 
@@ -367,14 +354,22 @@
     Transformed frame buffer
 */
 //#define QT_NO_QWS_TRANSFORMED
+#if defined(QT_NO_NETWORK)
 /*!
     Remote frame buffer (VNC)
 */
-#define QT_NO_QWS_VNC
+# ifndef QT_NO_QWS_VNC
+#  define QT_NO_QWS_VNC
+# endif
+#endif
 /*!
     1-bit monochrome
 */
 //#define QT_NO_QWS_DEPTH_1
+/*!
+    4-bit greyscale
+*/
+//#define QT_NO_QWS_DEPTH_4
 /*!
     4-bit VGA
 */
@@ -396,16 +391,32 @@
     15 or 16-bit color (define QT_QWS_DEPTH16_RGB as 555 for 15-bit)
 */
 //#define QT_NO_QWS_DEPTH_16
+/*!
+    24-bit color
+*/
+//#define QT_NO_QWS_DEPTH_24
 /*!
     32-bit color
 */
 //#define QT_NO_QWS_DEPTH_32
 
 /*!
-    Window manager
+    Window Manager
 */
 //#define QT_NO_QWS_MANAGER
 
+/*!
+    Window Manager Styles
+*/
+#define QT_NO_QWS_KDE2_WM_STYLE
+#if defined( QT_NO_QWS_MANAGER ) || defined( QT_NO_IMAGEIO_XPM )
+# define QT_NO_QWS_HYDRO_WM_STYLE
+# define QT_NO_QWS_BEOS_WM_STYLE
+# define QT_NO_QWS_KDE_WM_STYLE
+# define QT_NO_QWS_QPE_WM_STYLE
+# define QT_NO_QWS_WINDOWS_WM_STYLE
+#endif
+
 /*!
     Saving of fonts
 */
@@ -797,6 +808,10 @@
 	Motif-plus style
     */
 # define QT_NO_STYLE_MOTIFPLUS
+    /*!
+	Interlace style
+    */
+# define QT_NO_STYLE_INTERLACE
 #endif
 
 
