@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/tools/qtextcodec.cpp   2.2.2   edited 2000-11-03 $
+** $Id: qt/src/tools/qtextcodec.cpp   2.3.0   edited 2001-03-02 $
 **
 ** Implementation of QTextCodec class
 **
@@ -432,9 +432,12 @@ int QWindowsLocalCodec::heuristicContentMatch(const char* chars, int len) const
     int l = t.length();
     QCString mb = fromUnicode(t,l);
     int i=0;
-    while ( i < len )
+    while ( i < len ) {
         if ( chars[i] == mb[i] )
             i++;
+       else
+           break;
+    }
     return i;
 }
 
@@ -454,9 +457,15 @@ static const char * const iso8859_4locales[] = {
     "ee", "ee_EE", "lt", "lt_LT", "lv", "lv_LV", 0 };
 
 static const char * const iso8859_5locales[] = {
-    "bg", "bg_BG", "bulgarian", "mk", "mk_MK",
+    "mk", "mk_MK",
     "sp", "sp_YU", 0 };
 
+static const char * const cp_1251locales[] = {
+    "be", "be_BY", "bg", "bg_BG", "bulgarian", 0 };
+
+static const char * const pt_154locales[] = {
+    "ba_RU", "ky", "ky_KG", "kk", "kk_KZ", 0 };
+
 static const char * const iso8859_6locales[] = {
     "ar_AA", "ar_SA", "arabic", 0 };
 
@@ -554,6 +563,13 @@ QTextCodec* QTextCodec::codecForLocale()
 
     char * lang = qstrdup( getenv("LANG") );
 
+#ifdef _WS_X11_
+    // use LC_CTYPE if LANG is not available, as X11 uses it too, and we otherwise get inconsitencies between
+    // XmbLookupString and the Qt input mapper
+    if ( !lang || lang[0] == 0 ) 
+	lang = qstrdup( getenv("LC_CTYPE") );
+#endif
+    
     char * p = lang ? strchr( lang, '.' ) : 0;
     if ( !p || *p != '.' ) {
         // Some versions of setlocale return encoding, others not.
@@ -578,6 +594,8 @@ QTextCodec* QTextCodec::codecForLocale()
             // Use or codec disagree.
             localeMapper = codecForName( p+1 );
         }
+	if ( localeMapper && localeMapper->mibEnum() == 11 )
+	    localeMapper = codecForName( "ISO 8859-8-I" );
     }
     if ( !localeMapper || !(p && *p == '.') ) {
         // if there is none, we default to 8859-1
@@ -604,6 +622,10 @@ QTextCodec* QTextCodec::codecForLocale()
             localeMapper = codecForName( "ISO 8859-11" );
         else if ( try_locale_list( koi8_ulocales, lang ) )
             localeMapper = codecForName( "KOI8-U" );
+        else if ( try_locale_list( cp_1251locales, lang ) )
+            localeMapper = codecForName( "CP 1251" );
+        else if ( try_locale_list( pt_154locales, lang ) )
+            localeMapper = codecForName( "PT 154" );
          else if ( try_locale_list( probably_koi8_rlocales, lang ) )
             localeMapper = ru_RU_hack( lang );
         else if (!lang || !(localeMapper = codecForName(lang) ))
@@ -802,7 +824,15 @@ QCString QTextCodec::fromUnicode(const QString& uc) const
 */
 QString QTextCodec::toUnicode(const QByteArray& a, int len) const
 {
-    return toUnicode(a.data(),len);
+    int l = a.size();
+    // implicit conversions of QCStrings to QByteArrays used to give an additional 0 at the end.
+    // we should however not remove this 0 for utf16 encoded data.
+    if( l > 0 && a.data()[l - 1] == '\0' ) {
+	if ( mibEnum() != 1000 || a.data()[l-2] == 0)
+	    l--;
+    }
+    l = QMIN( l, len );
+    return toUnicode( a.data(), l );
 }
 
 /*!
@@ -810,7 +840,14 @@ QString QTextCodec::toUnicode(const QByteArray& a, int len) const
 */
 QString QTextCodec::toUnicode(const QByteArray& a) const
 {
-    return toUnicode(a.data(),a.size());
+    int l = a.size();
+    // implicit conversions of QCStrings to QByteArrays used to give an additional 0 at the end.
+    // we should however not remove this 0 for utf16 encoded data.
+    if( l > 0 && a.data()[l - 1] == '\0' ) {
+	if ( mibEnum() != 1000 || a.data()[l-2] == 0)
+	    l--;
+    }
+    return toUnicode( a.data(), l );
 }
 
 /*!
@@ -976,13 +1013,16 @@ public:
         const int maxlen=100;
         char line[maxlen];
         char esc='\\';
+        char comm='%';
         bool incmap = FALSE;
         while (iod->readLine(line,maxlen) > 0) {
             if (0==qstrnicmp(line,"<code_set_name>",15))
                 n = line+15;
-            else if (0==qstrnicmp(line,"<escape_char>",13))
+            else if (0==qstrnicmp(line,"<escape_char> ",14))
                 esc = line[14];
-            else if (0==qstrnicmp(line,"% alias ",8)) {
+            else if (0==qstrnicmp(line,"<comment_char> ",15))
+                comm = line[15];
+            else if (line[0]==comm && 0==qstrnicmp(line+1," alias ",7)) {
                 aliases.append(line+8);
             } else if (0==qstrnicmp(line,"CHARMAP",7)) {
                 if (!from_unicode_page) {
@@ -1004,45 +1044,54 @@ public:
                 char mb[maxmb+1];
                 int nmb=0;
 
-                while (*cursor && *cursor!=' ')
-                    cursor++;
-                while (*cursor && *cursor!=esc)
-                    cursor++;
-                byte = getByte(cursor);
+                while (*cursor) {
+                    if (cursor[0]=='<' && cursor[1]=='U' &&
+                        cursor[2]>='0' && cursor[2]<='9' &&
+                        cursor[3]>='0' && cursor[3]<='9') {
 
-                if ( *cursor == esc ) {
-                    if ( !to_unicode_multibyte ) {
-                        to_unicode_multibyte = new QMultiByteUnicodeTable[256];
-                        for (int i=0; i<256; i++) {
-                            to_unicode_multibyte[i].unicode = to_unicode[i];
-                            to_unicode_multibyte[i].multibyte = 0;
-                        }
-                        delete [] to_unicode;
-                        to_unicode = 0;
-                    }
-                    QMultiByteUnicodeTable* mbut = to_unicode_multibyte+byte;
-                    mb[nmb++] = byte;
-                    while ( nmb < maxmb && *cursor == esc ) {
-                        // Always at least once
+                        unicode = strtol(cursor+2,&cursor,16);
+
+                    } else if (*cursor==esc) {
 
-                        mbut->unicode = CHAINED;
                         byte = getByte(cursor);
-                        mb[nmb++] = byte;
-                        if (!mbut->multibyte) {
-                            mbut->multibyte =
-                                new QMultiByteUnicodeTable[256];
-                        }
-                        mbut = mbut->multibyte+byte;
-                        mb_unicode = & mbut->unicode;
-                    }
 
-                    if ( nmb > max_bytes_per_char )
-                        max_bytes_per_char = nmb;
+                        if ( *cursor == esc ) {
+                            if ( !to_unicode_multibyte ) {
+                                to_unicode_multibyte =
+                                    new QMultiByteUnicodeTable[256];
+                                for (int i=0; i<256; i++) {
+                                    to_unicode_multibyte[i].unicode =
+                                        to_unicode[i];
+                                    to_unicode_multibyte[i].multibyte = 0;
+                                }
+                                delete [] to_unicode;
+                                to_unicode = 0;
+                            }
+                            QMultiByteUnicodeTable* mbut =
+                                to_unicode_multibyte+byte;
+                            mb[nmb++] = byte;
+                            while ( nmb < maxmb && *cursor == esc ) {
+                                // Always at least once
+
+                                mbut->unicode = CHAINED;
+                                byte = getByte(cursor);
+                                mb[nmb++] = byte;
+                                if (!mbut->multibyte) {
+                                    mbut->multibyte =
+                                        new QMultiByteUnicodeTable[256];
+                                }
+                                mbut = mbut->multibyte+byte;
+                                mb_unicode = & mbut->unicode;
+                            }
+
+                            if ( nmb > max_bytes_per_char )
+                                max_bytes_per_char = nmb;
+                        }
+                    } else {
+                        cursor++;
+                    }
                 }
-                while (*cursor && (*cursor!='<' || cursor[1]!='U'))
-                    cursor++;
-                if ( *cursor )
-                    unicode = strtol(cursor+2,&cursor,16);
+
                 if (unicode >= 0 && unicode <= 0xffff)
                 {
                     QChar ch((ushort)unicode);
@@ -1590,6 +1639,23 @@ static struct {
 
     // $ for a in CP* ; do ( awk '/^0x[89ABCDEF]/{ print $1, $2 }' < $a ) | sort | sed -e 's/#UNDEF.*$/0xFFFD/' | cut -c6- | paste '-d ' - - - - - - - - | sed -e 's/ /, /g' -e 's/$/,/' -e '$ s/,$/} },/' -e '1 s/^/{ /' > ~/tmp/$a ; done
 
+    { "PT 154", 0,  // unknown mib
+        {     0x0496, 0x0492, 0x201A, 0x0493, 0x201E, 0x2026, 0x2020, 0x04AE,    
+	      0x04B2, 0x04AF, 0x04A0, 0x2039, 0x04A2, 0x049A, 0x04BA, 0x04B8,    
+	      0x0497, 0x2018, 0x2019, 0x201C, 0x201D, 0x2022, 0x2013, 0x2014,    
+	      0x04B3, 0x2122, 0x04A1, 0x203A, 0x04A3, 0x049B, 0x04BB, 0x04B9,    
+	      0x00A0, 0x040E, 0x045E, 0x0408, 0x04E8, 0x0498, 0x04B0, 0x00A7,
+	      0x0401, 0x00A9, 0x04D8, 0x00AB, 0x00AC, 0x00AD, 0x00AE, 0x049C,
+	      0x00B0, 0x04B1, 0x0406, 0x0456, 0x0499, 0x04E9, 0x00B6, 0x00B7,
+	      0x0451, 0x2116, 0x04D9, 0x00BB, 0x0458, 0x04AA, 0x04AB, 0x049D,
+              0x0410, 0x0411, 0x0412, 0x0413, 0x0414, 0x0415, 0x0416, 0x0417,
+              0x0418, 0x0419, 0x041A, 0x041B, 0x041C, 0x041D, 0x041E, 0x041F,
+              0x0420, 0x0421, 0x0422, 0x0423, 0x0424, 0x0425, 0x0426, 0x0427,
+              0x0428, 0x0429, 0x042A, 0x042B, 0x042C, 0x042D, 0x042E, 0x042F,
+              0x0430, 0x0431, 0x0432, 0x0433, 0x0434, 0x0435, 0x0436, 0x0437,
+              0x0438, 0x0439, 0x043A, 0x043B, 0x043C, 0x043D, 0x043E, 0x043F,
+              0x0440, 0x0441, 0x0442, 0x0443, 0x0444, 0x0445, 0x0446, 0x0447,
+              0x0448, 0x0449, 0x044A, 0x044B, 0x044C, 0x044D, 0x044E, 0x044F} },
     { "CP 874", 0, //### what is the mib?
         { 0x20AC, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0x2026, 0xFFFD, 0xFFFD,
               0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD,
@@ -1771,7 +1837,7 @@ static struct {
               0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD,
               0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD,
               0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD,
-              0xFFFD, 0x0E01, 0x0E02, 0x0E03, 0x0E04, 0x0E05, 0x0E06, 0x0E07,
+              0x00A0, 0x0E01, 0x0E02, 0x0E03, 0x0E04, 0x0E05, 0x0E06, 0x0E07,
               0x0E08, 0x0E09, 0x0E0A, 0x0E0B, 0x0E0C, 0x0E0D, 0x0E0E, 0x0E0F,
               0x0E10, 0x0E11, 0x0E12, 0x0E13, 0x0E14, 0x0E15, 0x0E16, 0x0E17,
               0x0E18, 0x0E19, 0x0E1A, 0x0E1B, 0x0E1C, 0x0E1D, 0x0E1E, 0x0E1F,
@@ -1851,6 +1917,8 @@ QCString QSimpleTextCodec::fromUnicode(const QString& uc, int& len ) const
             (*reverseMap)[i] = (char)i;
         for( ;i < m; i++ )
             (*reverseMap)[i] = '?';
+	// small hack to map nbsp to space for all simple codecs. Looks way better than a question mark.
+	(*reverseMap)[0xa0] = ' ';
         for( i=128; i<256; i++ ) {
             int u = unicodevalues[forwardIndex].values[i-128];
             if ( u < m )
@@ -1903,7 +1971,10 @@ int QSimpleTextCodec::heuristicNameMatch(const char* hint) const
 	if( !i )
 	    i = simpleHeuristicNameMatch("tis-620", hint);
 	if( i ) return i;
-    }
+    } else if (( hint[0]=='m' ) && ( QCString(hint) == "microsoft-cp1251" ))
+         return QTextCodec::heuristicNameMatch("cp1251");
+    else if (( hint[0]=='w' ) && ( QCString(hint) == "windows-1251" ))
+         return QTextCodec::heuristicNameMatch("cp1251")-1;
     return QTextCodec::heuristicNameMatch(hint);
 }
 
