@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qt/src/widgets/qcombobox.cpp   2.2.2   edited 2000-10-15 $
+** $Id: qt/src/widgets/qcombobox.cpp   2.3.0   edited 2001-03-02 $
 **
 ** Implementation of QComboBox widget class
 **
@@ -95,10 +95,9 @@
   The two constructors create identical-looking combos in Windows
   style.
 
-  Read-only combo boxes can contain pixmaps as well as texts; the
-  insert() and changeItem() functions are suitably overloaded.  If you
-  try to insert a pixmap in a read-write combo box, QComboBox simply
-  ignores you.  For read-write combo boxes, the function clearEdit()
+  Combo boxes can contain pixmaps as well as texts; the
+  insert() and changeItem() functions are suitably overloaded.  For 
+  read-write combo boxes, the function clearEdit()
   is provided, to clear the displayed string without changing the
   combo box' contents.
 
@@ -703,6 +702,15 @@ void QComboBox::removeItem( int index )
 	}
 	currentChanged();
     }
+    else {
+        if ( !d->ed ) {
+            if (d->current < cnt - 1)
+                setCurrentItem( d->current );
+            else
+                setCurrentItem( d->current - 1 );
+        }
+    }
+
 }
 
 
@@ -806,7 +814,7 @@ void QComboBox::changeItem( const QString &t, int index )
 
 void QComboBox::changeItem( const QPixmap &im, int index )
 {
-    if ( d->ed != 0 || !checkIndex( "changeItem", name(), count(), index ) )
+    if ( !checkIndex( "changeItem", name(), count(), index ) )
 	return;
     if ( d->usingListBox() )
 	d->listBox()->changeItem( im, index );
@@ -818,17 +826,12 @@ void QComboBox::changeItem( const QPixmap &im, int index )
 
 /*!
   Replaces the item at position \e index with a pixmap plus text.
-  If the combo box is writable, the pixmap is ignored.
-
+  
   \sa insertItem()
 */
 
 void QComboBox::changeItem( const QPixmap &im, const QString &t, int index )
 {
-    if ( d->ed != 0 ) {
-	changeItem( t, index );
-	return;
-    }
     if ( !checkIndex( "changeItem", name(), count(), index ) )
 	return;
     if ( d->usingListBox() )
@@ -1345,7 +1348,8 @@ void QComboBox::popup()
 	d->listBox()->viewport()->installEventFilter( this );
 	d->mouseWasInsidePopup = FALSE;
 	d->listBox()->resize( width(),
-			    listHeight( d->listBox(), d->sizeLimit ) + 2 );
+			    listHeight( d->listBox(), d->sizeLimit ) +
+			    d->listBox()->frameWidth() * 2 );
 	QWidget *desktop = QApplication::desktop();
 	int sw = desktop->width();			// screen width
 	int sh = desktop->height();			// screen height
@@ -1367,9 +1371,10 @@ void QComboBox::popup()
 
 	d->listBox()->move( x,y );
 	d->listBox()->raise();
+	bool block = d->listBox()->signalsBlocked();
 	d->listBox()->blockSignals( TRUE );
 	d->listBox()->setCurrentItem( d->listBox()->item( d->current ) );
-	d->listBox()->blockSignals( FALSE );
+	d->listBox()->blockSignals( block );
 	d->listBox()->setAutoScrollBar( TRUE );
 
 #ifndef QT_NO_EFFECTS
@@ -1497,7 +1502,7 @@ bool QComboBox::eventFilter( QObject *object, QEvent *event )
 		    if ( it.length() >= ct.length() ) {
 			it.truncate( ct.length() );
 			int itlen = text( i ).length();
-			if ( (it == ct || it.lower() == ct )
+			if ( ( it.lower() == ct.lower() )
 			    && itlen < foundLength ) {
 			    foundAt = i;
 			    foundLength = text( i ).length();
@@ -1508,8 +1513,10 @@ bool QComboBox::eventFilter( QObject *object, QEvent *event )
 		}
 		if ( foundAt > -1 ) {
 		    it = text( foundAt );
-		    d->ed->validateAndSet( it, ct.length(),
+            if ( !d->ed->hasMarkedText() ) {
+                d->ed->validateAndSet( it, ct.length(),
 					   ct.length(), it.length() );
+            }
 		}
 	    }
 	}
@@ -1753,8 +1760,12 @@ void QComboBox::returnPressed()
     }
 
     if ( doInsert ) {
-	if ( count() == d->maxCount )
-	    removeItem( count() - 1 );
+	if ( insertionPolicy() != NoInsertion ) {
+	    int cnt = count();
+	    while ( cnt >= d->maxCount ) {
+		removeItem( --cnt );
+	    }
+	}
 	
 	switch ( insertionPolicy() ) {
 	case AtCurrent:
@@ -1936,12 +1947,15 @@ void QComboBox::setEditText( const QString &newText )
 
   The combo box uses the list of items as candidates for completion.
 
+  Note: This will only work on editable combo boxes, so make the combo
+  box editable before you call this function or it will not work.
+
   \sa autoCompletion() setEditText()
 */
 
 void QComboBox::setAutoCompletion( bool enable )
 {
-    d->useCompletion = enable && (d->ed != 0);
+    d->useCompletion = enable;
     d->completeNow = FALSE;
 }
 
