@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qlcdnumber.cpp   2.2.2   edited 2000-11-01 $
+** $Id: qt/src/widgets/qlcdnumber.cpp   2.3.0   edited 2001-01-26 $
 **
 ** Implementation of QLCDNumber class
 **
@@ -187,12 +187,15 @@ static QString double2string( double num, int base, int ndigits, bool *oflow )
 	}
 	s = int2string( (int)num, base, ndigits, 0 );
     } else {					// decimal base
-	s.sprintf( "%*.*g", ndigits, ndigits, num );
-	int i = s.find('e');
-	if ( i > 0 && s[i+1]=='+' ) {
-	    s[i] = ' ';
-	    s[i+1] = 'e';
-	}
+	int nd = ndigits;
+	do {
+	    s.sprintf( "%*.*g", ndigits, nd, num );
+	    int i = s.find('e');
+	    if ( i > 0 && s[i+1]=='+' ) {
+		s[i] = ' ';
+		s[i+1] = 'e';
+	    }
+	} while (nd-- && (int)s.length() > ndigits);
     }
     if ( oflow )
 	*oflow = (int)s.length() > ndigits;
