@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qt/src/widgets/qlistbox.cpp   2.2.2   edited 2000-10-16 $
+** $Id: qt/src/widgets/qlistbox.cpp   2.3.0   edited 2001-03-01 $
 **
 ** Implementation of QListBox widget class
 **
@@ -1297,6 +1297,7 @@ void QListBox::removeItem( int index )
 
 void QListBox::clear()
 {
+    bool blocked = signalsBlocked();
     blockSignals( TRUE );
     d->clearing = TRUE;
     d->current = 0;
@@ -1319,7 +1320,7 @@ void QListBox::clear()
     d->mouseMoveColumn = -1;
     d->selectable.clear();
     clearSelection();
-    blockSignals( FALSE );
+    blockSignals( blocked );
     triggerUpdate( TRUE );
     d->last = 0;
     d->clearing = FALSE;
@@ -3733,9 +3734,10 @@ void QListBoxPrivate::findItemByName( const QString &text )
 	listBox->setCurrentItem( item );
 	if ( selectionMode == QListBox::Extended ) {
 	    bool changed = FALSE;
+	    bool block = listBox->signalsBlocked();
 	    listBox->blockSignals( TRUE );
 	    listBox->selectAll( FALSE );
-	    listBox->blockSignals( FALSE );
+	    listBox->blockSignals( block );
 	    if ( !item->s && item->isSelectable() ) {
 		changed = TRUE;
 		item->s = TRUE;
@@ -3905,6 +3907,8 @@ void QListBox::sort( bool ascending )
     if ( count() == 0 )
 	return;
 
+    d->cache = 0;
+
     QListBoxPrivate::SortableItem *items = new QListBoxPrivate::SortableItem[ count() ];
 
     QListBoxItem *item = d->head;
@@ -3962,9 +3966,10 @@ void QListBox::handleItemChange( QListBoxItem *old, bool shift, bool control )
 	    selectRange( d->selectAnchor ? d->selectAnchor : old,
 			 d->current, FALSE, TRUE, d->selectAnchor ? TRUE : FALSE );
 	} else {
+	    bool block = signalsBlocked();
 	    blockSignals( TRUE );
 	    selectAll( FALSE );
-	    blockSignals( FALSE );
+	    blockSignals( block );
 	    setSelected( d->current, TRUE );
 	}
     } else if ( d->selectionMode == Multi ) {
