@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qpopupmenu.cpp   2.2.2   edited 2000-11-02 $
+** $Id: qt/src/widgets/qpopupmenu.cpp   2.3.0   edited 2001-02-15 $
 **
 ** Implementation of QPopupMenu class
 **
@@ -421,6 +421,8 @@ void QPopupMenu::popup( const QPoint &pos, int indexAtPoint )
     QWidget *desktop = QApplication::desktop();
     int sw = desktop->width();			// screen width
     int sh = desktop->height();			// screen height
+    int sx = desktop->x();			// screen pos
+    int sy = desktop->y();
     int x  = pos.x();
     int y  = pos.y();
     if ( indexAtPoint > 0 )			// don't subtract when < 0
@@ -433,20 +435,21 @@ void QPopupMenu::popup( const QPoint &pos, int indexAtPoint )
 	    x = mouse.x()-w;
 	if ( y+h > sh )
 	    y = mouse.y()-h;
-	if ( x < 0 )
+	if ( x < sx )
 	    x = mouse.x();
-	if ( y < 0 )
-	    y = mouse.y();
-    } else {
-	if ( x+w > sw )				// the complete widget must
-	    x = sw - w;				//   be visible
-	if ( y+h > sh )
+	if ( y < sy )
 	    y = sh - h;
-	if ( x < 0 )
-	    x = 0;
-	if ( y < 0 )
-	    y = 0;
     }
+
+    if ( x+w > sw )				// the complete widget must
+	x = sw - w;				//   be visible
+    if ( y+h > sh )
+	y = sh - h;
+    if ( x < sx )
+	x = sx;
+    if ( y < sy )
+	y = sy;
+
     move( x, y );
     motion=0;
     actItem = -1;
@@ -456,7 +459,7 @@ void QPopupMenu::popup( const QPoint &pos, int indexAtPoint )
     int vGuess = QEffects::DownScroll;
     if ( snapToMouse && ( x + w/2 < mouse.x() ) ||
 	( parentMenu && parentMenu->isPopupMenu &&
-	( x + w/2 < ((QPopupMenu*)parentMenu)->mapToGlobal( ((QPopupMenu*)parentMenu)->pos() ).x() ) ) )
+	( x + w/2 < ((QPopupMenu*)parentMenu)->x() ) ) )
 	hGuess = QEffects::LeftScroll;
     if ( snapToMouse && ( y + h/2 < mouse.y() ) ||
 	( parentMenu && parentMenu->isMenuBar &&
@@ -625,7 +628,7 @@ void QPopupMenu::hidePopups()
 {
     if ( preventAnimation )
 	QTimer::singleShot( 10, this, SLOT(allowAnimation()) );
-    preventAnimation = TRUE;	
+    preventAnimation = TRUE;
 
     QMenuItemListIt it(*mitems);
     register QMenuItem *mi;
@@ -951,7 +954,7 @@ void QPopupMenu::updateAccel( QWidget *parent )
 	int k = mi->key();
 	if ( k ) {
 	    int id = autoaccel->insertItem( k, mi->id() );
-#ifndef QT_NO_WHATSTHIS	    
+#ifndef QT_NO_WHATSTHIS
 	    autoaccel->setWhatsThis( id, mi->whatsThis() );
 #endif
 	}
@@ -1234,7 +1237,7 @@ void QPopupMenu::mouseReleaseEvent( QMouseEvent *e )
 	    bool b = QWhatsThis::inWhatsThisMode();
 #else
 	    const bool b = FALSE;
-#endif	    
+#endif
 	if ( !mi->isEnabled() ) {
 #ifndef QT_NO_WHATSTHIS
 	    if ( b ) {
@@ -1243,7 +1246,7 @@ void QPopupMenu::mouseReleaseEvent( QMouseEvent *e )
 		byeMenuBar();
 		actSig( mi->id(), b);
 	    }
-#endif	    
+#endif
 	} else 	if ( popup ) {
 	    popup->setFirstItemActive();
 	} else {				// normal menu item
@@ -1327,6 +1330,22 @@ void QPopupMenu::mouseMoveEvent( QMouseEvent *e )
 
 void QPopupMenu::keyPressEvent( QKeyEvent *e )
 {
+    if (mouseBtDn && actItem >= 0) {
+	if (e->key() == Key_Shift ||
+	    e->key() == Key_Control ||
+	    e->key() == Key_Alt)
+	    return;
+
+	QMenuItem *mi = mitems->at(actItem);
+	int modifier = (((e->state() & ShiftButton) ? SHIFT : 0) |
+			((e->state() & ControlButton) ? CTRL : 0) |
+			((e->state() & AltButton) ? ALT : 0));
+
+	if (mi)
+	    setAccel(modifier + e->key(), mi->id());  ;
+	return;
+    }
+
     QMenuItem  *mi = 0;
     QPopupMenu *popup;
     int dy = 0;
@@ -1388,8 +1407,10 @@ void QPopupMenu::keyPressEvent( QKeyEvent *e )
 	    hidePopups();
 	    if ( singleSingleShot )
 		singleSingleShot->stop();
-	    popup->setFirstItemActive();
-	    subMenuTimer();
+            // ### The next two lines were switched to fix the problem with the first item of the
+            // submenu not being highlighted...any reason why they should have been the other way??
+            subMenuTimer();
+            popup->setFirstItemActive();
 	    break;
 	} else if ( actItem == -1 && ( parentMenu && !parentMenu->isMenuBar )) {
 	    dy = 1;
@@ -1429,7 +1450,7 @@ void QPopupMenu::keyPressEvent( QKeyEvent *e )
 	    bool b = QWhatsThis::inWhatsThisMode();
 #else
 	    const bool b = FALSE;
-#endif	    
+#endif
 	    if ( mi->isEnabled() || b ) {
 		active_popup_menu = this;
 		actSig( mi->id(), b );
@@ -1451,7 +1472,7 @@ void QPopupMenu::keyPressEvent( QKeyEvent *e )
 	    QWhatsThis::leaveWhatsThisMode( mi->whatsThis(), mapToGlobal( r.bottomLeft()) );
 	}
 	//fall-through!
-#endif	
+#endif
     default:
 	ok_key = FALSE;
 
@@ -1490,7 +1511,7 @@ void QPopupMenu::keyPressEvent( QKeyEvent *e )
 		bool b = QWhatsThis::inWhatsThisMode();
 #else
 		const bool b = FALSE;
-#endif	    
+#endif
 		if ( mi->isEnabled() || b ) {
 		    active_popup_menu = this;
 		    actSig( mi->id(), b );
@@ -1998,7 +2019,7 @@ void QPopupMenu::activateItemAt( int index )
 	    bool b = QWhatsThis::inWhatsThisMode();
 #else
 	    const bool b = FALSE;
-#endif	    
+#endif
 	    if ( !mi->isEnabled() ) {
 #ifndef QT_NO_WHATSTHIS
 		if ( b ) {
