@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qsgistyle.cpp   2.2.2   edited 2000-10-12 $
+** $Id: qt/src/widgets/qsgistyle.cpp   2.3.0   edited 2001-02-06 $
 **
 ** Implementation of Motif-like style class
 **
@@ -62,8 +62,6 @@ static const int sgiSepHeight		= 1;	// separator item height
 static const int sgiItemHMargin		= 3;	// menu item hor text margin
 static const int sgiItemVMargin 	= 2;	// menu item ver text margin
 static const int sgiArrowHMargin	= 6;	// arrow horizontal margin
-static const int sgiTabSpacing		= 12;	// space between text and tab
-static const int sgiCheckMarkHMargin	= 2;	// horiz. margins of check mark
 static const int sgiCheckMarkSpace	= 20;
 
 static bool sliderMoving		= FALSE;
@@ -234,17 +232,21 @@ QSGIStyle::polish( QWidget* w )
     }
 
     if ( w->inherits("QButton") || w->inherits("QSlider") || w->inherits("QScrollBar") ) {
-	w->installEventFilter( this );
-	w->setMouseTracking( TRUE );
-	if ( w->inherits("QScrollBar") )
-	    w->setBackgroundMode( QWidget::NoBackground );
+        w->installEventFilter( this );
+        w->setMouseTracking( TRUE );
+        if ( w->inherits("QToolButton") )
+            w->setBackgroundMode( QWidget::PaletteBackground );
+        if ( w->inherits("QScrollBar") )
+            w->setBackgroundMode( QWidget::NoBackground );
     } else if ( w->inherits("QMenuBar") ) {
-	((QFrame*) w)->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
-	w->setBackgroundMode( QWidget::PaletteBackground );
+        ((QFrame*) w)->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
+        w->setBackgroundMode( QWidget::PaletteBackground );
     } else if ( w->inherits("QPopupMenu") ) {
-	((QFrame*) w)->setLineWidth( defaultFrameWidth() + 1 );
+        ((QFrame*) w)->setLineWidth( defaultFrameWidth() + 1 );
     } else if ( w->inherits("QToolBar") ) {
-	w->setBackgroundMode( QWidget::PaletteBackground );
+        w->setBackgroundMode( QWidget::PaletteBackground );
+    } else if ( w->inherits("QToolBarSeparator") ) {
+        w->setBackgroundMode( QWidget::PaletteBackground );
     }
 }
 
@@ -298,6 +300,8 @@ void
 QSGIStyle::drawPanel( QPainter*p, int x, int y, int w, int h, const QColorGroup &g,
 		      bool sunken, int lineWidth, const QBrush* fill )
 {
+    if( w < 0 ) w = 0;
+    if( h < 0 ) h = 0;
     QMotifStyle::drawPanel( p, x, y, w, h, g, sunken, ( w > lineWidth && h > lineWidth ) ? lineWidth : 1, fill );
     if ( lineWidth <= 1 )
 	return;
@@ -1104,7 +1108,7 @@ QSGIStyle::drawPopupPanel( QPainter *p, int x, int y, int w, int h,
 {
     if (lineWidth == 3 && w > 3 && h > 3 )
         drawBevelButton( p, x, y,  w, h, g, FALSE, fill );
-    else 
+    else
 	QMotifStyle::drawPopupPanel( p, x, y, w, h, g, lineWidth, fill );
 }
 
@@ -1356,7 +1360,7 @@ QSGIStyle::eventFilter( QObject* o, QEvent* e )
 
 	    if ( lastWidget && o == lastWidget &&
 		lastWidget->testWState( WState_Created )) {
-		if ( lastWidgetPalette ) {    
+		if ( lastWidgetPalette ) {
 		    lastWidget->setPalette( *lastWidgetPalette );
 		    delete lastWidgetPalette;
 		    lastWidgetPalette = 0;
