@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qsplitter.cpp   2.2.2   edited 2000-10-11 $
+** $Id: qt/src/widgets/qsplitter.cpp   2.3.0   edited 2001-03-02 $
 **
 **  Splitter widget
 **
@@ -619,7 +619,9 @@ void QSplitter::getRange( int id, int *min, int *max )
     int i;
     for ( i = 0; i < id; i++ ) {
 	QSplitterLayoutStruct *s = data->list.at(i);
-	if ( s->isSplitter ) {
+	if ( s->wid->isHidden() ) {
+	    //ignore
+	} else if ( s->isSplitter ) {
 	    minB += s->sizer;
 	    maxB += s->sizer;
 	} else {
@@ -629,7 +631,9 @@ void QSplitter::getRange( int id, int *min, int *max )
     }
     for ( i = id; i < n; i++ ) {
 	QSplitterLayoutStruct *s = data->list.at(i);
-	if ( s->isSplitter ) {
+	if ( s->wid->isHidden() ) {
+	    //ignore
+	} else 	if ( s->isSplitter ) {
 	    minA += s->sizer;
 	    maxA += s->sizer;
 	} else {
@@ -1044,6 +1048,8 @@ bool QSplitter::isHidden( QWidget *w ) const
 
   Giving the values to setSizes() will give a splitter with the same
   layout as this one.
+
+  \sa setSizes()
 */
 
 QValueList<int> QSplitter::sizes() const
@@ -1073,6 +1079,8 @@ QValueList<int> QSplitter::sizes() const
 
   If \a list contains too few values, the result is undefined
   but the program will still be well-behaved.
+
+  \sa sizes()
 */
 
 void QSplitter::setSizes( QValueList<int> list )
