@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qtabbar.cpp   2.2.2   edited 2000-10-11 $
+** $Id: qt/src/widgets/qtabbar.cpp   2.3.0   edited 2001-02-13 $
 **
 ** Implementation of QTab and QTabBar classes
 **
@@ -602,7 +602,6 @@ void QTabBar::show()
     //  ensures that one tab is selected.
     QTab * t = l->last();
     QWidget::show();
-    updateArrowButtons();
 
     if ( t )
 	emit selected( t->id );
@@ -914,7 +913,7 @@ void QTabBar::scrollTabs()
     for ( QTab* t = lstatic->first(); t; t = lstatic->next() ) {
 	if ( t->r.left() < 0 && t->r.right() > 0 )
 	    left = t;
-	if ( t->r.left() < d->leftB->x() )
+	if ( t->r.left() < d->leftB->x()+2 )
 	    right = t;
     }
 
