@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qtabwidget.cpp   2.2.2   edited 2000-10-11 $
+** $Id: qt/src/widgets/qtabwidget.cpp   2.3.0   edited 2001-03-01 $
 **
 ** Implementation of QTabWidget class
 **
@@ -344,10 +344,11 @@ void QTabWidget::changeTab( QWidget *w, const QString &label)
     d->tabs->layoutTabs();
 
     int ct = d->tabs->currentTab();
+    bool block = d->tabs->signalsBlocked();
     d->tabs->blockSignals( TRUE );
     d->tabs->setCurrentTab( 0 );
     d->tabs->setCurrentTab( ct );
-    d->tabs->blockSignals( FALSE );
+    d->tabs->blockSignals( block );
 
     d->tabs->update();
     setUpLayout();
@@ -372,10 +373,11 @@ void QTabWidget::changeTab( QWidget *w, const QIconSet& iconset, const QString &
     d->tabs->layoutTabs();
 
     int ct = d->tabs->currentTab();
+    bool block = d->tabs->signalsBlocked();
     d->tabs->blockSignals( TRUE );
     d->tabs->setCurrentTab( 0 );
     d->tabs->setCurrentTab( ct );
-    d->tabs->blockSignals( FALSE );
+    d->tabs->blockSignals( block );
 
     d->tabs->update();
     setUpLayout();
@@ -484,6 +486,8 @@ int QTabWidget::currentPageIndex() const
 }
 
 /*! Sets the page with index \a id as current page.
+
+    Note that \e id is not the index that is specified when you insert a tab
 */
 
 void QTabWidget::setCurrentPage( int id )
