@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qtoolbar.cpp   2.2.2   edited 2000-10-13 $
+** $Id: qt/src/widgets/qtoolbar.cpp   2.3.0   edited 2001-02-17 $
 **
 ** Implementation of QToolBar class
 **
@@ -55,7 +55,7 @@
 class QArrowWidget : public QWidget
 {
     Q_OBJECT
-    
+
 public:
     QArrowWidget( Qt::Orientation o, QWidget *parent ) : QWidget( parent, "qt_arrow_widget" ), orient( o ) {}
 
@@ -291,7 +291,12 @@ void QToolBar::init()
 
     bl = new QBoxLayout( this, orientation() == Vertical
 			? QBoxLayout::Down : QBoxLayout::LeftToRight,
-			style() == WindowsStyle ? 2 : 1, 0 );
+#ifdef _WS_QWS_
+			1,
+#else
+			style() == WindowsStyle ? 2 : 1,
+#endif
+			0 );
     boxLayout()->setAutoAdd( TRUE );
     if ( !mw || mw->toolBarsMovable() )
 	boxLayout()->addSpacing( 9 );
@@ -346,6 +351,27 @@ void QToolBar::addSeparator()
     (void) new QToolBarSeparator( orientation(), this, "tool bar separator" );
 }
 
+/*!\reimp
+*/
+
+void QToolBar::styleChange( QStyle& /*old*/ )
+{
+    QObjectList *childs = queryList( "QWidget" );
+    if ( childs ) {
+        QObject *ob = 0;
+	for ( ob = childs->first(); ob; ob = childs->next() ) {
+            if ( ob->inherits( "QToolButton" ) ) {
+                QToolButton* w = (QToolButton*)ob;
+                w->setStyle( &style() );
+            } else if ( ob->inherits( "QToolBarSeparator" ) ) {
+                QToolBarSeparator* w = (QToolBarSeparator*)ob;
+                w->setStyle( &style() );
+            }
+        }
+    }
+    delete childs;
+}
+    
 
 /*!  Sets this toolbar to organize its content vertically if \a
   newOrientation is \c Vertical and horizontally if \a newOrientation
@@ -894,9 +920,10 @@ void QToolBar::emulateButtonClicked()
 	emit d->button->released();
 	emit d->button->clicked();
     }
-    d->button = 0;
-
-    QTimer::singleShot( 0, this, SLOT( updateArrowStuff() ) );
+    if ( d ) { // we might got deleted, so check to be sure
+	d->button = 0;
+	QTimer::singleShot( 0, this, SLOT( updateArrowStuff() ) );
+    }
 }
 
 void QToolBar::paintToolBar()
