@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qwhatsthis.cpp   2.2.2   edited 2000-10-11 $
+** $Id: qt/src/widgets/qwhatsthis.cpp   2.3.0   edited 2001-01-26 $
 **
 ** Implementation of QWhatsThis class
 **
@@ -523,8 +523,10 @@ void QWhatsThisPrivate::say_helper(QWidget* widget,const QPoint& ppos,bool init)
 			 ) : QApplication::desktop()->width() )
 		- w;
 
-	if ( x < 0 )
-	    x = 0;
+	int sx = QApplication::desktop()->x();
+	int sy = QApplication::desktop()->y();
+	if ( x < sx )
+	    x = sx;
 
 	int y;
 	if ( widget && h > widget->height() + 16 ) {
@@ -542,8 +544,8 @@ void QWhatsThisPrivate::say_helper(QWidget* widget,const QPoint& ppos,bool init)
 				 pos.y() + widget->height())
 			    ) : QApplication:: desktop()->height() )
 		- h;
-	if ( y < 0 )
-	    y = 0;
+	if ( y < sy )
+	    y = sy;
 	
 	whatsThat->setGeometry( x, y, w + shadowWidth, h + shadowWidth );
 	whatsThat->show();
