@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qwindowsstyle.cpp   2.2.2   edited 2000-10-11 $
+** $Id: qt/src/widgets/qwindowsstyle.cpp   2.3.0   edited 2001-01-26 $
 **
 ** Implementation of Windows-like style class
 **
@@ -304,7 +304,7 @@ QWindowsStyle::indicatorSize() const
 
 void QWindowsStyle::drawExclusiveIndicator( QPainter* p,
 				   int x, int y, int w, int h, const QColorGroup &g,
-				   bool on, bool down, bool /* enabled */ )
+				   bool on, bool down, bool enabled )
 {
 
     static const QCOORD pts1[] = {		// dark lines
@@ -337,10 +337,9 @@ void QWindowsStyle::drawExclusiveIndicator( QPainter* p,
     p->drawPolyline( a );
     a.setPoints( QCOORDARRLEN(pts5), pts5 );
     a.translate( x, y );
-    QColor fillColor = down ? g.button() : g.base();
+    QColor fillColor = ( down || !enabled ) ? g.button() : g.base();
     p->setPen( fillColor );
-    p->setBrush( down ?  g.brush( QColorGroup::Button ) :
-			 g.brush( QColorGroup::Base ) ) ;
+    p->setBrush( fillColor  ) ;
     p->drawPolygon( a );
     if ( on ) {
 	p->setPen( NoPen );
