@@ -184,12 +184,19 @@ void FormWindow::paintGrid( QWidget *w, QPaintEvent *e )
 {
     if ( !mainWindow()->showGrid() )
 	return;
+    int x = 0, y = 0, jmax = 0;
     QPainter p( w );
     p.setClipRegion( e->rect() );
     p.setPen( colorGroup().foreground() );
-    for ( int i = 0; i < w->width() / mainWindow()->grid().x() + 20; ++i ) {
-	for ( int j = 0; j < w->height() / mainWindow()->grid().y() + 20; ++j )
-	    p.drawPoint( i * mainWindow()->grid().x(), j * mainWindow()->grid().y() );
+    jmax = w->height() / mainWindow()->grid().y() + 20;
+    int end = w->width() / mainWindow()->grid().x() + 20;
+    for ( int i = 0; i < end; ++i ) {
+	y = 0;
+	for ( int j = 0; j < jmax; ++j ) {
+	    p.drawPoint( x, y );
+	    y += mainWindow()->grid().y();
+	}
+	x += mainWindow()->grid().x();
     }
 }
 
