@@ -713,9 +713,12 @@ void Resource::saveObjectProperties( QObject *w, QTextStream &ts, int indent )
     bool inLayout = w != formwindow->mainContainer() && !copying && w->isWidgetType() && ( (QWidget*)w )->parentWidget() &&
 		    WidgetFactory::layoutType( ( (QWidget*)w )->parentWidget() ) != WidgetFactory::NoLayout;
 	
-    QStringList::Iterator it = changed.begin();
-    for ( ; it != changed.end(); ++it ) {
-	const QMetaProperty* p = w->metaObject()->property( *it, TRUE );
+
+    QStrList lst = w->metaObject()->propertyNames( !w->inherits( "Spacer" ) );
+    for ( QListIterator<char> it( lst ); it.current(); ++it ) {
+	if ( changed.find( QString::fromLatin1( it.current() ) ) == changed.end() )
+	    continue;
+	const QMetaProperty* p = w->metaObject()->property( it.current(), TRUE );
 	if ( !p || !p->stored( w ) || ( inLayout && qstrcmp( p->name(), "geometry" ) == 0 ) )
 	    continue;
 	if ( w->inherits( "QLabel" ) && qstrcmp( p->name(), "pixmap" ) == 0 &&
@@ -728,14 +731,14 @@ void Resource::saveObjectProperties( QObject *w, QTextStream &ts, int indent )
 	    ts << " stdset=\"1\"";
 	ts << ">" << endl;
 	indent++;
-	ts << makeIndent( indent ) << "<name>" << *it << "</name>" << endl;
+	ts << makeIndent( indent ) << "<name>" << it.current() << "</name>" << endl;
 	
 	if ( p->isSetType() ) {
-	    saveSetProperty( w, *it, QVariant::nameToType( p->type() ), ts, indent );
+	    saveSetProperty( w, it.current(), QVariant::nameToType( p->type() ), ts, indent );
 	} else if ( p->isEnumType() ) {
-	    saveEnumProperty( w, *it, QVariant::nameToType( p->type() ), ts, indent );
+	    saveEnumProperty( w, it.current(), QVariant::nameToType( p->type() ), ts, indent );
 	} else {
-	    saveProperty( w, *it, w->property( p->name() ), QVariant::nameToType( p->type() ), ts, indent );
+	    saveProperty( w, it.current(), w->property( p->name() ), QVariant::nameToType( p->type() ), ts, indent );
 	}
 	indent--;
 	ts << makeIndent( indent ) << "</property>" << endl;
