@@ -18,15 +18,18 @@
 *****************************************************************************/
 
 #include "qvfbview.h"
-#include <qvfbhdr_qws.h>
-#include <qwscommand_qws.h>	    // for QTE_PIPE
+#include "qvfbhdr.h"
 
+#define QTE_PIPE "QtEmbedded-%1"
+
+#include <qapplication.h>
 #include <qimage.h>
 #include <qbitmap.h>
 #include <qtimer.h>
 #include <qwmatrix.h>
 #include "qanimationwriter.h"
 
+#include <stdlib.h>
 #include <unistd.h>
 #include <sys/ipc.h>
 #include <sys/types.h>
@@ -40,7 +43,7 @@ QVFbView::QVFbView( int display_id, int w, int h, int d, QWidget *parent,
 		    const char *name, uint flags )
     : QScrollView( parent, name, flags ), lockId(-1)
 {
-    displayId = display_id;
+    displayid = display_id;
     viewport()->setMouseTracking( TRUE );
     viewport()->setFocusPolicy( StrongFocus );
     zoom = 1;
@@ -48,7 +51,9 @@ QVFbView::QVFbView( int display_id, int w, int h, int d, QWidget *parent,
 
     switch ( d ) {
 	case 1:
+	case 4:
 	case 8:
+	case 16:
 	case 32:
 	    break;
 	
@@ -125,6 +130,27 @@ QVFbView::~QVFbView()
     unlink( keyboardPipe );
 }
 
+int QVFbView::displayId() const
+{
+    return displayid;
+}
+
+int QVFbView::displayWidth() const
+{
+    return hdr->width;
+}
+
+int QVFbView::displayHeight() const
+{
+    return hdr->height;
+}
+
+int QVFbView::displayDepth() const
+{
+    return hdr->depth;
+}
+
+
 
 void QVFbView::setZoom( double z )
 {
@@ -132,6 +158,9 @@ void QVFbView::setZoom( double z )
 	zoom = z;
 	setDirty(QRect(0,0,hdr->width,hdr->height));
 	resizeContents( int(hdr->width*z), int(hdr->height*z) );
+	updateGeometry();
+	qApp->sendPostedEvents();
+	topLevelWidget()->adjustSize();
 	drawScreen();
     }
 }
@@ -144,7 +173,13 @@ void QVFbView::setRate( int r )
 
 void QVFbView::initLock()
 {
-    QString pipe = QString( QTE_PIPE ).arg( displayId );
+    QString username = "unknown";
+    const char *logname = getenv("LOGNAME");
+    if ( logname )
+	username = logname;
+    QString dataDir = "/tmp/qtembedded-" + username;
+
+    QString pipe = dataDir + "/" + QString( QTE_PIPE ).arg( displayid );
     int semkey = ftok( pipe.latin1(), 'd' );
     lockId = semget( semkey, 0, 0 );
 }
@@ -203,17 +238,14 @@ void QVFbView::timeout()
 {
     lock();
     if ( animation ) {
-	    // ### Could use img from drawScreen, with offset, but we do it
-	    // ### this way for now to exercise QAnimationWriter.
 	    QRect r( hdr->update );
 	    r = r.intersect( QRect(0, 0, hdr->width, hdr->height ) );
 	    if ( r.isEmpty() ) {
 		animation->appendBlankFrame();
 	    } else {
-		QImage img( data + hdr->dataoffset + r.y() * hdr->linestep,
-			hdr->width, r.height(), hdr->depth, hdr->clut,
-			hdr->depth == 1 ? 0 : 256, QImage::LittleEndian );
-		animation->appendFrame(img,QPoint(0,r.y()));
+		int l;
+		QImage img = getBuffer( r, l );
+		animation->appendFrame(img,QPoint(r.x(),r.y()));
 	    }
     }
     if ( hdr->dirty ) {
@@ -222,6 +254,78 @@ void QVFbView::timeout()
     unlock();
 }
 
+QImage QVFbView::getBuffer( const QRect &r, int &leading ) const
+{
+    switch ( hdr->depth ) {
+      case 16: {
+	static unsigned char *imgData = 0;
+	if ( !imgData ) {
+	    int bpl = ((hdr->width*32+31)/32)*4;
+	    imgData = new unsigned char [ bpl * hdr->height ];
+	}
+	QImage img( imgData, r.width(), r.height(), 32, 0, 0, QImage::IgnoreEndian );
+	for ( int row = 0; row < r.height(); row++ ) {
+	    QRgb *dptr = (QRgb*)img.scanLine( row );
+	    ushort *sptr = (ushort*)(data + hdr->dataoffset + (r.y()+row)*hdr->linestep);
+	    sptr += r.x();
+	    for ( int col=0; col < r.width(); col++ ) {
+		ushort s = *sptr++;
+		*dptr++ = qRgb((s>>11)*255/31,((s>>5)&0x3f)*255/63,(s&0x1f)*255/31);
+	    }
+	}
+	leading = 0;
+	return img;
+      }
+      case 4: {
+	static unsigned char *imgData = 0;
+	if ( !imgData ) {
+	    int bpl = ((hdr->width*8+31)/32)*4;
+	    imgData = new unsigned char [ bpl * hdr->height ];
+	}
+	QImage img( imgData, r.width(), r.height(), 8, hdr->clut, 16,
+		    QImage::IgnoreEndian );
+	for ( int row = 0; row < r.height(); row++ ) {
+	    unsigned char *dptr = img.scanLine( row );
+	    unsigned char *sptr = data + hdr->dataoffset + (r.y()+row)*hdr->linestep;
+	    sptr += r.x()/2;
+	    int col = 0;
+	    if ( r.x() & 1 ) {
+		*dptr++ = *sptr++ >> 4;
+		col++;
+	    }
+	    for ( ; col < r.width()-1; col+=2 ) {
+		unsigned char s = *sptr++;
+		*dptr++ = s & 0x0f;
+		*dptr++ = s >> 4;
+	    }
+	    if ( !(r.right() & 1) )
+		*dptr = *sptr & 0x0f;
+	}
+	leading = 0;
+	return img;
+      }
+      case 32: {
+	leading = r.x();
+	return QImage( data + hdr->dataoffset + r.y() * hdr->linestep,
+		    hdr->width, r.height(), hdr->depth, 0,
+		    0, QImage::LittleEndian );
+      }
+      case 8: {
+	leading = r.x();
+	return QImage( data + hdr->dataoffset + r.y() * hdr->linestep,
+		    hdr->width, r.height(), hdr->depth, hdr->clut,
+		    256, QImage::LittleEndian );
+      }
+      case 1: {
+	leading = r.x();
+	return QImage( data + hdr->dataoffset + r.y() * hdr->linestep,
+		    hdr->width, r.height(), hdr->depth, hdr->clut,
+		    0, QImage::LittleEndian );
+      }
+    }
+    return QImage();
+}
+
 void QVFbView::drawScreen()
 {
     QPainter p( viewport() );
@@ -232,7 +336,7 @@ void QVFbView::drawScreen()
     QRect r( hdr->update );
     hdr->dirty = FALSE;
     hdr->update = QRect();
-//    qDebug( "update %d, %d, %dx%d", r.y(), r.x(), r.width(), r.height() );
+    // qDebug( "update %d, %d, %dx%d", r.y(), r.x(), r.width(), r.height() );
     r = r.intersect( QRect(0, 0, hdr->width, hdr->height ) );
     if ( !r.isEmpty() )  {
 	if ( int(zoom) != zoom ) {
@@ -243,9 +347,8 @@ void QVFbView::drawScreen()
 	    r.rRight() = QMIN(r.right(),hdr->width-1);
 	    r.rBottom() = QMIN(r.bottom(),hdr->height-1);
 	}
-	QImage img( data + hdr->dataoffset + r.y() * hdr->linestep,
-		    hdr->width, r.height(), hdr->depth, hdr->clut,
-		    hdr->depth == 1 ? 0 : 256, QImage::LittleEndian );
+	int leading;
+	QImage img( getBuffer( r, leading ) );
 	QPixmap pm;
 	if ( zoom == 1 ) {
 	    pm.convertFromImage( img );
@@ -261,7 +364,7 @@ void QVFbView::drawScreen()
 	p.setPen( black );
 	p.setBrush( white );
 	p.drawPixmap( int(r.x()*zoom), int(r.y()*zoom), pm,
-			int(r.x()*zoom), 0, pm.width(), pm.height() );
+			int(leading*zoom), 0, pm.width(), pm.height() );
     } else {
 	unlock();
     }
@@ -325,18 +428,19 @@ void QVFbView::keyPressEvent( QKeyEvent *e )
 
 void QVFbView::keyReleaseEvent( QKeyEvent *e )
 {
-    sendKeyboardData(e->text()[0].unicode(), e->key(), 
+    sendKeyboardData(e->ascii(), e->key(), 
 		     e->state()&(ShiftButton|ControlButton|AltButton),
 		     FALSE, e->isAutoRepeat());
 }
 
 
-void QVFbView::saveAs( const QString& filename )
+QImage QVFbView::image() const
 {
-    QImage img( data + hdr->dataoffset,
-		hdr->width, hdr->height, hdr->depth, hdr->clut,
-		256, QImage::LittleEndian );
-    img.save(filename,"PNG");
+    ((QVFbView*)this)->lock();
+    int l;
+    QImage r = getBuffer( QRect(0, 0, hdr->width, hdr->height), l ).copy();
+    ((QVFbView*)this)->unlock();
+    return r;
 }
 
 void QVFbView::startAnimation( const QString& filename )
